# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# pylint: disable=line-too-long

import unittest
from datetime import datetime

from pontos.nvd.models.cpe import CPE, ReferenceType
from tests.nvd import get_cpe_data


class CPETestCase(unittest.TestCase):
    def test_required_only(self):
        cpe = CPE.from_dict(get_cpe_data())

        self.assertEqual(
            cpe.cpe_name,
            "cpe:2.3:o:microsoft:windows_10_22h2:-:*:*:*:*:*:arm64:*",
        )
        self.assertEqual(
            cpe.cpe_name_id, "9BAECDB2-614D-4E9C-9936-190C30246F03"
        )
        self.assertFalse(cpe.deprecated)
        self.assertEqual(
            cpe.last_modified, datetime(2022, 12, 9, 18, 15, 16, 973000)
        )
        self.assertEqual(cpe.created, datetime(2022, 12, 9, 16, 20, 6, 943000))

        self.assertEqual(cpe.titles, [])
        self.assertEqual(cpe.refs, [])
        self.assertEqual(cpe.deprecated_by, [])

    def test_titles(self):
        cpe = CPE.from_dict(
            get_cpe_data(
                {
                    "titles": [
                        {
                            "title": "Microsoft Windows 10 22h2 on ARM64",
                            "lang": "en",
                        }
                    ],
                }
            )
        )

        self.assertEqual(len(cpe.titles), 1)

        title = cpe.titles[0]
        self.assertEqual(title.title, "Microsoft Windows 10 22h2 on ARM64")
        self.assertEqual(title.lang, "en")

    def test_refs(self):
        cpe = CPE.from_dict(
            get_cpe_data(
                {
                    "refs": [
                        {
                            "ref": "https://learn.microsoft.com/en-us/windows/release-health/release-information",
                            "type": "Version",
                        }
                    ],
                }
            )
        )

        self.assertEqual(len(cpe.refs), 1)

        reference = cpe.refs[0]
        self.assertEqual(
            reference.ref,
            "https://learn.microsoft.com/en-us/windows/release-health/release-information",
        )
        self.assertEqual(reference.type, ReferenceType.VERSION)

    def test_deprecated(self):
        cpe = CPE.from_dict(
            get_cpe_data(
                {
                    "deprecated": True,
                    "deprecated_by": [
                        {
                            "cpe_name": "cpe:2.3:o:microsoft:windows_10_22h2:-:*:*:*:*:*:x64:*",
                            "cpe_name_id": "A09335E2-B42F-4820-B487-57A4BF0CEE98",
                        }
                    ],
                }
            )
        )

        self.assertTrue(cpe.deprecated)
        self.assertEqual(len(cpe.deprecated_by), 1)

        deprecated_by = cpe.deprecated_by[0]
        self.assertEqual(
            deprecated_by.cpe_name,
            "cpe:2.3:o:microsoft:windows_10_22h2:-:*:*:*:*:*:x64:*",
        )
        self.assertEqual(
            deprecated_by.cpe_name_id, "A09335E2-B42F-4820-B487-57A4BF0CEE98"
        )



# Copyright (C) 2023 Greenbone Networks GmbH
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import re
from typing import Any, Optional, Tuple

from semver import VersionInfo

from .._calculator import VersionCalculator
from ..errors import VersionError
from ..version import Version
from ._scheme import VersioningScheme

# Note: This regex currently support any kind of
# word-number combination for pre releases
_PRE_RELEASE_REGEXP = re.compile(
    r"^(?P<name>[a-zA-Z]+)(?P<version>0|[1-9][0-9]*)"
    r"(?:-(?P<extra>[a-zA-Z]+)(?P<extra_version>0|[1-9][0-9]*))?$"
)


class SemanticVersion(Version):
    """
    A Version implementation based on
    `Semantic Versioning<https://semver.org/>`_
    """

    def __init__(
        self,
        version: str,
    ) -> None:
        self._version_info = VersionInfo.parse(version)
        self._parse_build()
        self._parse_pre_release()

    def _parse_pre_release(self) -> None:
        self._dev = None
        self._pre_release = None

        if self._version_info.prerelease:
            match = _PRE_RELEASE_REGEXP.match(self._version_info.prerelease)
            if not match:
                raise VersionError(
                    f"Invalid prerelease {self._version_info.prerelease} in "
                    f"{self._version_info}"
                )

            name = match.group("name")
            version = int(match.group("version"))

            if name == "dev":
                self._dev = version
            else:
                self._pre_release = (
                    name,
                    version,
                )

            extra = match.group("extra")
            if extra == "dev":
                if name == "dev":
                    raise VersionError(
                        f"Invalid prerelease {self._version_info.prerelease} "
                        f"in {self._version_info}"
                    )
                self._dev = int(match.group("extra_version"))

    def _parse_build(self) -> None:
        self._build = None
        if self._version_info.build:
            match = _PRE_RELEASE_REGEXP.match(self._version_info.build)
            if match:
                self._build = (
                    match.group("name"),
                    int(match.group("version")),
                )

    @property
    def pre(self) -> Optional[Tuple[str, int]]:
        """The pre-release segment of the version."""
        return self._pre_release

    @property
    def dev(self) -> Optional[int]:
        """The development number of the version."""
        return self._dev

    @property
    def local(self) -> Optional[Tuple[str, int]]:
        """The local version segment of the version."""
        return self._build

    @property
    def is_pre_release(self) -> bool:
        """
        Whether this version is a pre-release (alpha, beta, release candidate).
        """
        return self._pre_release is not None

    @property
    def is_dev_release(self) -> bool:
        """Whether this version is a development release."""
        return self._dev is not None

    @property
    def is_alpha_release(self) -> bool:
        """Whether this version is a alpha release."""
        return self.is_pre_release and self.pre[0] == "alpha"

    @property
    def is_beta_release(self) -> bool:
        """Whether this version is a beta release."""
        return self.is_pre_release and self.pre[0] == "beta"

    @property
    def is_release_candidate(self) -> bool:
        """Whether this version is a release candidate."""
        return self.is_pre_release and self.pre[0] == "rc"

    @property
    def major(self) -> int:
        """The first item of :attr:`release` or ``0`` if unavailable."""
        return self._version_info.major

    @property
    def minor(self) -> int:
        """The second item of :attr:`release` or ``0`` if unavailable."""
        return self._version_info.minor

    @property
    def patch(self) -> int:
        """The third item of :attr:`release` or ``0`` if unavailable."""
        return self._version_info.patch

    def __eq__(self, other: Any) -> bool:
        if other is None:
            return False
        if isinstance(other, str):
            # allow to compare against "current" for now
            return False
        if not isinstance(other, Version):
            raise ValueError(f"Can't compare {type(self)} with {type(other)}")
        if not isinstance(other, type(self)):
            other = self.from_version(other)

        return (
            self._version_info == other._version_info
            and self._version_info.build == other._version_info.build
        )

    def __ne__(self, other: Any) -> bool:
        if other is None:
            return True
        if isinstance(other, str):
            # allow to compare against "current" for now
            return True
        if not isinstance(other, Version):
            raise ValueError(f"Can't compare {type(self)} with {type(other)}")
        if not isinstance(other, type(self)):
            other = self.from_version(other)

        return (
            self._version_info != other._version_info
            or self._version_info.build != other._version_info.build
        )

    def __gt__(self, other: Any) -> bool:
        if not isinstance(other, Version):
            raise ValueError(f"Can't compare {type(self)} with {type(other)}")
        if not isinstance(other, type(self)):
            other = self.from_version(other)

        if self._version_info.to_tuple()[:3] > other._version_info[:3]:
            return True
        if self._version_info.to_tuple()[:3] < other._version_info[:3]:
            return False

        # major, minor and patch are equal
        if self.is_dev_release:
            if not other.is_pre_release and not other.is_dev_release:
                return False
            if not self.is_pre_release and other.is_pre_release:
                return False
            if not self.is_pre_release:
                return self.dev > other.dev

            if self.is_pre_release:
                if other.is_dev_release and self.pre == other.pre:
                    return self.dev > other.dev
                return self.pre > other.pre

        # not a dev release
        if self.is_pre_release:
            if not other.is_pre_release and not other.is_dev_release:
                return False

            if other.is_pre_release:
                if other.is_dev_release:
                    return self.pre >= other.pre
                return self.pre > other.pre

            # other is a dev release
            return True

        if other.is_dev_release or other.is_pre_release:
            return True

        # both are equal
        return False

    def __str__(self) -> str:
        """A string representation of the version"""
        return str(self._version_info)

    @classmethod
    def from_string(cls, version: str) -> "SemanticVersion":
        """
        Create a version from a version string

        Args:
            version: Version string to parse

        Raises:
            VersionError: If the version string is invalid.

        Returns:
            A new version instance
        """
        try:
            return cls(version)
        except ValueError as e:
            raise VersionError(e) from None

    @classmethod
    def from_version(cls, version: "Version") -> "SemanticVersion":
        """
        Convert a version (if necessary)

        This method can be used to convert version instances from different
        versioning schemes.
        """

        if isinstance(version, cls):
            return version

        version_local = (
            f"+{version.local[0]}{version.local[1]}" if version.local else ""
        )
        if version.is_dev_release:
            if not version.pre:
                return cls.from_string(
                    f"{version.major}."
                    f"{version.minor}."
                    f"{version.patch}"
                    f"-dev{version.dev}"
                    f"{version_local}"
                )

            return cls.from_string(
                f"{version.major}."
                f"{version.minor}."
                f"{version.patch}"
                f"-{version.pre[0]}{version.pre[1]}"
                f"-dev{version.dev}"
            )
        if version.is_alpha_release:
            return cls.from_string(
                f"{version.major}."
                f"{version.minor}."
                f"{version.patch}"
                f"-alpha{version.pre[1]}"
                f"{version_local}"
            )
        if version.is_beta_release:
            return cls.from_string(
                f"{version.major}."
                f"{version.minor}."
                f"{version.patch}"
                f"-beta{version.pre[1]}"
                f"{version_local}"
            )
        if version.is_pre_release:
            return cls.from_string(
                f"{version.major}."
                f"{version.minor}."
                f"{version.patch}"
                f"-{version.pre[0]}{version.pre[1]}"
                f"{version_local}"
            )

        return cls.from_string(str(version))


# pylint: disable=protected-access
class SemanticVersionCalculator(VersionCalculator):
    version_cls = SemanticVersion

    @classmethod
    def next_dev_version(cls, current_version: Version) -> Version:
        """
        Get the next development version from a valid version
        """
        if current_version.is_dev_release:
            if current_version.pre:
                return cls.version_from_string(
                    f"{current_version.major}."
                    f"{current_version.minor}."
                    f"{current_version.patch}"
                    f"-{current_version.pre[0]}{current_version.pre[1]}"
                    f"-dev{current_version.dev + 1}"
                )
            return cls.version_from_string(
                f"{current_version.major}."
                f"{current_version.minor}."
                f"{current_version.patch}"
                f"-dev{current_version.dev + 1}"
            )

        if current_version.is_pre_release:
            return cls.version_from_string(
                f"{current_version.major}."
                f"{current_version.minor}."
                f"{current_version.patch}-"
                f"{current_version.pre[0]}{current_version.pre[1] + 1}-dev1"
            )

        return cls.version_from_string(
            f"{current_version.major}."
            f"{current_version.minor}."
            f"{current_version.patch + 1}"
            "-dev1"
        )

    @classmethod
    def next_alpha_version(cls, current_version: Version) -> Version:
        """
        Get the next alpha version from a valid version
        """
        if current_version.is_dev_release:
            if current_version.pre:
                if current_version.pre[0] == "alpha":
                    return cls.version_from_string(
                        f"{current_version.major}."
                        f"{current_version.minor}."
                        f"{current_version.patch}"
                        f"-alpha{current_version.pre[1]}"
                    )
                return cls.version_from_string(
                    f"{current_version.major}."
                    f"{current_version.minor}."
                    f"{current_version.patch + 1}-alpha1"
                )
            return cls.version_from_string(
                f"{current_version.major}."
                f"{current_version.minor}."
                f"{current_version.patch}-alpha1"
            )
        if current_version.is_alpha_release:
            return cls.version_from_string(
                f"{current_version.major}."
                f"{current_version.minor}."
                f"{current_version.patch}"
                f"-alpha{current_version.pre[1] + 1}"
            )
        return cls.version_from_string(
            f"{current_version.major}."
            f"{current_version.minor}."
            f"{current_version.patch + 1}"
            "-alpha1"
        )

    @classmethod
    def next_beta_version(cls, current_version: Version) -> Version:
        """
        Get the next alpha version from a valid version
        """
        if current_version.is_dev_release and current_version.pre:
            if current_version.pre[0] == "beta":
                return cls.version_from_string(
                    f"{current_version.major}."
                    f"{current_version.minor}."
                    f"{current_version.patch}"
                    f"-beta{current_version.pre[1]}"
                )
            if current_version.pre[0] == "rc":
                return cls.version_from_string(
                    f"{current_version.major}."
                    f"{current_version.minor}."
                    f"{current_version.patch + 1}-beta1"
                )
        if current_version.is_dev_release or current_version.is_alpha_release:
            return cls.version_from_string(
                f"{current_version.major}."
                f"{current_version.minor}."
                f"{current_version.patch}"
                "-beta1"
            )
        if current_version.is_beta_release:
            return cls.version_from_string(
                f"{current_version.major}."
                f"{current_version.minor}."
                f"{current_version.patch}"
                f"-beta{current_version.pre[1] + 1}"
            )
        return cls.version_from_string(
            f"{current_version.major}."
            f"{current_version.minor}."
            f"{current_version.patch + 1}"
            "-beta1"
        )

    @classmethod
    def next_release_candidate_version(
        cls, current_version: Version
    ) -> Version:
        """
        Get the next release candidate version from a valid version
        """
        if current_version.is_dev_release:
            if current_version.pre and current_version.pre[0] == "rc":
                return cls.version_from_string(
                    f"{current_version.major}."
                    f"{current_version.minor}."
                    f"{current_version.patch}"
                    f"-rc{current_version.pre[1]}"
                )
            return cls.version_from_string(
                f"{current_version.major}."
                f"{current_version.minor}."
                f"{current_version.patch}-rc1"
            )
        if current_version.is_alpha_release or current_version.is_beta_release:
            return cls.version_from_string(
                f"{current_version.major}."
                f"{current_version.minor}."
                f"{current_version.patch}-rc1"
            )
        if current_version.is_release_candidate:
            return cls.version_from_string(
                f"{current_version.major}."
                f"{current_version.minor}."
                f"{current_version.patch}"
                f"-rc{current_version.pre[1] + 1}"
            )
        return cls.version_from_string(
            f"{current_version.major}."
            f"{current_version.minor}."
            f"{current_version.patch + 1}"
            "-rc1"
        )


class SemanticVersioningScheme(VersioningScheme):
    version_cls = SemanticVersion
    version_calculator_cls = SemanticVersionCalculator
    name = "SemVer"



# Copyright (C) 2020-2022 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import unittest
from unittest.mock import patch

from pontos.git.git import Git
from pontos.version.errors import VersionError
from pontos.version.helper import (
    get_last_release_version,
    get_last_release_versions,
)
from pontos.version.schemes import (
    PEP440VersioningScheme,
    SemanticVersioningScheme,
)

parse_version = PEP440VersioningScheme.parse_version
Version = PEP440VersioningScheme.version_cls


class GetLastReleaseVersionsTestCase(unittest.TestCase):
    @patch("pontos.version.helper.Git", spec=Git)
    def test_get_last_release_versions(self, git_mock):
        git_interface = git_mock.return_value
        git_interface.list_tags.return_value = ["1", "2", "3.55"]

        it = get_last_release_versions(parse_version)

        version = next(it)
        self.assertEqual(version, Version("3.55"))

        version = next(it)
        self.assertEqual(version, Version("2"))

        version = next(it)
        self.assertEqual(version, Version("1"))

        with self.assertRaises(StopIteration):
            next(it)

    @patch("pontos.version.helper.Git", spec=Git)
    def test_get_no_release_versions(self, git_mock):
        git_interface = git_mock.return_value
        git_interface.list_tags.return_value = []

        it = get_last_release_versions(parse_version)
        with self.assertRaises(StopIteration):
            next(it)

    @patch("pontos.version.helper.Git", spec=Git)
    def test_get_last_release_versions_with_git_prefix(self, git_mock):
        git_interface = git_mock.return_value
        git_interface.list_tags.return_value = ["v1", "v2", "v3.55"]

        it = get_last_release_versions(parse_version, git_tag_prefix="v")

        version = next(it)
        self.assertEqual(version, Version("3.55"))

        version = next(it)
        self.assertEqual(version, Version("2"))

        version = next(it)
        self.assertEqual(version, Version("1"))

        with self.assertRaises(StopIteration):
            next(it)

    @patch("pontos.version.helper.Git", spec=Git)
    def test_get_last_release_versions_ignore_pre_releases(self, git_mock):
        git_interface = git_mock.return_value
        git_interface.list_tags.return_value = [
            "1",
            "2",
            "3.55a1",
            "3.56.dev1",
            "4.0.0rc1",
            "4.0.1b1",
        ]
        it = get_last_release_versions(parse_version, ignore_pre_releases=True)

        version = next(it)
        self.assertEqual(version, Version("2"))

        version = next(it)
        self.assertEqual(version, Version("1"))

        with self.assertRaises(StopIteration):
            next(it)

    @patch("pontos.version.helper.Git", spec=Git)
    def test_get_last_release_versions_no_non_pre_release(self, git_mock):
        git_interface = git_mock.return_value
        git_interface.list_tags.return_value = [
            "3.55a1",
            "3.56.dev1",
            "4.0.0rc1",
            "4.0.1b1",
        ]

        it = get_last_release_versions(parse_version, ignore_pre_releases=True)

        with self.assertRaises(StopIteration):
            next(it)

    @patch("pontos.version.helper.Git", spec=Git)
    def test_invalid_version(self, git_mock):
        git_interface = git_mock.return_value
        git_interface.list_tags.return_value = [
            "1.0.0",
            "3.55a1",
            "2.0.0",
        ]

        it = get_last_release_versions(SemanticVersioningScheme.parse_version)
        next(it)

        with self.assertRaisesRegex(
            VersionError, "3.55a1 is not valid SemVer string"
        ):
            next(it)


class GetLastReleaseVersionTestCase(unittest.TestCase):
    @patch("pontos.version.helper.Git", spec=Git)
    def test_get_last_release_version(self, git_mock):
        git_interface = git_mock.return_value
        git_interface.list_tags.return_value = ["1", "2", "3.55"]
        self.assertEqual(
            get_last_release_version(parse_version), Version("3.55")
        )

    @patch("pontos.version.helper.Git", spec=Git)
    def test_get_no_release_version(self, git_mock):
        git_interface = git_mock.return_value
        git_interface.list_tags.return_value = []
        self.assertIsNone(get_last_release_version(parse_version))

    @patch("pontos.version.helper.Git", spec=Git)
    def test_get_last_release_version_with_git_prefix(self, git_mock):
        git_interface = git_mock.return_value
        git_interface.list_tags.return_value = ["v1", "v2", "v3.55"]
        self.assertEqual(
            get_last_release_version(parse_version, git_tag_prefix="v"),
            Version("3.55"),
        )

    @patch("pontos.version.helper.Git", spec=Git)
    def test_get_last_release_version_ignore_pre_releases(self, git_mock):
        git_interface = git_mock.return_value
        git_interface.list_tags.return_value = [
            "1",
            "2",
            "3.55a1",
            "3.56.dev1",
            "4.0.0rc1",
            "4.0.1b1",
        ]
        self.assertEqual(
            get_last_release_version(parse_version, ignore_pre_releases=True),
            Version("2"),
        )

    @patch("pontos.version.helper.Git", spec=Git)
    def test_get_last_release_version_no_non_pre_release(self, git_mock):
        git_interface = git_mock.return_value
        git_interface.list_tags.return_value = [
            "3.55a1",
            "3.56.dev1",
            "4.0.0rc1",
            "4.0.1b1",
        ]
        self.assertIsNone(
            get_last_release_version(parse_version, ignore_pre_releases=True)
        )

    @patch("pontos.version.helper.Git", spec=Git)
    def test_get_last_release_version_tag_name(self, git_mock):
        git_interface = git_mock.return_value
        git_interface.list_tags.return_value = [
            "4.0.0rc1",
            "4.0.1b1",
        ]
        self.assertEqual(
            get_last_release_version(parse_version, tag_name="4.0.*"),
            Version("4.0.1b1"),
        )

    @patch("pontos.version.helper.Git", spec=Git)
    def test_invalid_version(self, git_mock):
        git_interface = git_mock.return_value
        git_interface.list_tags.return_value = ["1", "2", "3.55a1"]

        with self.assertRaisesRegex(
            VersionError, "3.55a1 is not valid SemVer string"
        ):
            get_last_release_version(SemanticVersioningScheme.parse_version)

    @patch("pontos.version.helper.Git", spec=Git)
    def test_success_with_invalid_version(self, git_mock):
        git_interface = git_mock.return_value
        git_interface.list_tags.return_value = [
            "1.0.0",
            "2.0.0",
            "3.55a1",
            "4.0.0",
        ]

        version = get_last_release_version(
            SemanticVersioningScheme.parse_version
        )
        self.assertEqual(
            version, SemanticVersioningScheme.parse_version("4.0.0")
        )



# Copyright (C) 2023 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# pylint: disable=line-too-long

import unittest

from lxml import etree

from pontos.testing import temp_directory, temp_file
from pontos.version import VersionError
from pontos.version.commands import JavaVersionCommand
from pontos.version.schemes import PEP440VersioningScheme


class GetCurrentJavaVersionCommandTestCase(unittest.TestCase):
    def test_get_current_version(self):
        content = """<?xml version="1.0" encoding="UTF-8"?>
        <project xmlns="http://maven.apache.org/POM/4.0.0" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:schemaLocation="http://maven.apache.org/POM/4.0.0 https://maven.apache.org/xsd/maven-4.0.0.xsd">
        <modelVersion>4.0.0</modelVersion><groupId>net.greenbone.umbrella</groupId>
        <artifactId>msspadminservice</artifactId><version>2023.5.3</version></project>"""
        with temp_file(content, name="pom.xml", change_into=True):
            cmd = JavaVersionCommand(PEP440VersioningScheme)
            version = cmd.get_current_version()

            self.assertEqual(
                version, PEP440VersioningScheme.parse_version("2023.5.3")
            )

    def test_no_project_file(self):
        with temp_directory(change_into=True), self.assertRaisesRegex(
            VersionError, ".* file not found."
        ):
            cmd = JavaVersionCommand(PEP440VersioningScheme)
            cmd.get_current_version()

    def test_no_package_version(self):
        content = """<?xml version="1.0" encoding="UTF-8"?>
        <project xmlns="http://maven.apache.org/POM/4.0.0" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
                xsi:schemaLocation="http://maven.apache.org/POM/4.0.0 https://maven.apache.org/xsd/maven-4.0.0.xsd">
            <modelVersion>4.0.0</modelVersion><groupId>net.greenbone.umbrella</groupId>
            <artifactId>msspadminservice</artifactId></project>"""
        with temp_file(
            content, name="pom.xml", change_into=True
        ), self.assertRaisesRegex(VersionError, "Version tag missing in"):
            cmd = JavaVersionCommand(PEP440VersioningScheme)
            cmd.get_current_version()

    def test_no_valid_xml_in_pom(self):
        content = "<"
        with temp_file(
            content, name="pom.xml", change_into=True
        ), self.assertRaisesRegex(
            VersionError, "StartTag: invalid element name,"
        ):
            cmd = JavaVersionCommand(PEP440VersioningScheme)
            cmd.get_current_version()

        content = "<foo><bar/>"
        with temp_file(
            content, name="pom.xml", change_into=True
        ), self.assertRaisesRegex(
            VersionError, "Premature end of data in tag foo"
        ):
            cmd = JavaVersionCommand(PEP440VersioningScheme)
            cmd.get_current_version()


class UpdateJavaVersionCommandTestCase(unittest.TestCase):
    def test_update_version_file(self):
        content = """<?xml version="1.0" encoding="UTF-8"?>
        <project xmlns="http://maven.apache.org/POM/4.0.0" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:schemaLocation="http://maven.apache.org/POM/4.0.0 https://maven.apache.org/xsd/maven-4.0.0.xsd">
        <modelVersion>4.0.0</modelVersion><groupId>net.greenbone.umbrella</groupId>
        <artifactId>msspadminservice</artifactId><version>2023.5.3</version></project>"""

        with temp_file(content, name="pom.xml", change_into=True) as temp:
            cmd = JavaVersionCommand(PEP440VersioningScheme)
            cmd.get_current_version()
            updated = cmd.update_version(
                PEP440VersioningScheme.parse_version("2023.6.0")
            )

            self.assertEqual(
                updated.previous,
                PEP440VersioningScheme.parse_version("2023.5.3"),
            )
            self.assertEqual(
                updated.new, PEP440VersioningScheme.parse_version("2023.6.0")
            )
            self.assertEqual(updated.changed_files, [temp.resolve()])

            with temp.open(mode="r", encoding="utf-8") as fp:
                fake_pom = etree.parse(fp).getroot()

            self.assertEqual(fake_pom.find("{*}version").text, "2023.6.0")

    def test_update_version_develop(self):
        content = """<?xml version="1.0" encoding="UTF-8"?>
        <project xmlns="http://maven.apache.org/POM/4.0.0" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:schemaLocation="http://maven.apache.org/POM/4.0.0 https://maven.apache.org/xsd/maven-4.0.0.xsd">
        <modelVersion>4.0.0</modelVersion><groupId>net.greenbone.umbrella</groupId>
        <artifactId>msspadminservice</artifactId><version>2023.5.3</version></project>"""

        with temp_file(content, name="pom.xml", change_into=True) as temp:
            cmd = JavaVersionCommand(PEP440VersioningScheme)
            new_version = PEP440VersioningScheme.parse_version("2023.6.0.dev1")
            updated = cmd.update_version(new_version)

            self.assertEqual(
                updated.previous,
                PEP440VersioningScheme.parse_version("2023.5.3"),
            )
            self.assertEqual(
                updated.new,
                PEP440VersioningScheme.parse_version("2023.6.0.dev1"),
            )
            self.assertEqual(updated.changed_files, [temp.resolve()])

            with temp.open(mode="r", encoding="utf-8") as fp:
                fake_pom = etree.parse(fp).getroot()

            self.assertEqual(fake_pom.find("{*}version").text, "2023.6.0.dev1")

    def test_no_update(self):
        content = """<?xml version="1.0" encoding="UTF-8"?>
        <project xmlns="http://maven.apache.org/POM/4.0.0" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:schemaLocation="http://maven.apache.org/POM/4.0.0 https://maven.apache.org/xsd/maven-4.0.0.xsd">
        <modelVersion>4.0.0</modelVersion><groupId>net.greenbone.umbrella</groupId>
        <artifactId>msspadminservice</artifactId><version>2023.5.3</version></project>"""

        with temp_file(content, name="pom.xml", change_into=True) as temp:
            cmd = JavaVersionCommand(PEP440VersioningScheme)
            updated = cmd.update_version(
                PEP440VersioningScheme.parse_version("2023.5.3")
            )

            self.assertEqual(
                updated.previous,
                PEP440VersioningScheme.parse_version("2023.5.3"),
            )
            self.assertEqual(
                updated.new, PEP440VersioningScheme.parse_version("2023.5.3")
            )
            self.assertEqual(updated.changed_files, [])

            with temp.open(mode="r", encoding="utf-8") as fp:
                fake_pom = etree.parse(fp).getroot()

            self.assertEqual(fake_pom.find("{*}version").text, "2023.5.3")

    def test_forced_update(self):
        content = """<?xml version="1.0" encoding="UTF-8"?>
        <project xmlns="http://maven.apache.org/POM/4.0.0" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:schemaLocation="http://maven.apache.org/POM/4.0.0 https://maven.apache.org/xsd/maven-4.0.0.xsd">
        <modelVersion>4.0.0</modelVersion><groupId>net.greenbone.umbrella</groupId>
        <artifactId>msspadminservice</artifactId><version>2023.5.3</version></project>"""

        with temp_file(content, name="pom.xml", change_into=True) as temp:
            cmd = JavaVersionCommand(PEP440VersioningScheme)
            updated = cmd.update_version(
                PEP440VersioningScheme.parse_version("2023.5.3"), force=True
            )

            self.assertEqual(
                updated.previous,
                PEP440VersioningScheme.parse_version("2023.5.3"),
            )
            self.assertEqual(
                updated.new, PEP440VersioningScheme.parse_version("2023.5.3")
            )
            self.assertEqual(updated.changed_files, [temp.resolve()])

            with temp.open(mode="r", encoding="utf-8") as fp:
                fake_pom = etree.parse(fp).getroot()

            self.assertEqual(fake_pom.find("{*}version").text, "2023.5.3")



# Copyright (C) 2022 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from dataclasses import dataclass, field
from datetime import datetime
from enum import Enum
from typing import List, Optional

from pontos.models import Model

__all__ = (
    "DeprecatedBy",
    "ReferenceType",
    "Reference",
    "Title",
    "CPE",
)


@dataclass
class Title(Model):
    """
    A CPE title

    Attributes:
        title: The actual title
        lang: Language of the title
    """

    title: str
    lang: str


class ReferenceType(Enum):
    """
    A CPE reference type

    Attributes:
        ADVISORY: The reference is an advisory
        CHANGELOG: The reference is a changelog
        PRODUCT: The reference is a product
        PROJECT: The reference is a project
        VENDOR: The reference is a vendor
        VERSION: The reference is version
    """

    ADVISORY = "Advisory"
    CHANGELOG = "Change Log"
    PRODUCT = "Product"
    PROJECT = "Project"
    VENDOR = "Vendor"
    VERSION = "Version"


@dataclass
class Reference(Model):
    """
    A CPE reference

    Attributes:
        ref: The content of the reference
        type: The type of the reference
    """

    ref: str
    type: Optional[ReferenceType] = None


@dataclass
class DeprecatedBy(Model):
    """
    A CPE is deprecated by another CPE

    Attributes:
        cpe_name: Name of the CPE that deprecates this CPE
        cpe_name_id: ID of the CPE that deprecates this CPE
    """

    cpe_name: Optional[str] = None
    cpe_name_id: Optional[str] = None


@dataclass
class CPE(Model):
    """
    Represents a CPE

    Attributes:
        cpe_name: The name of the CPE
        cpe_name_id: ID of the CPE
        deprecated: True if the CPE is deprecated
        last_modified: Last modification date of the CPE
        created: Creation date of the CPE
        titles: List of titles for the CPE
        refs: References to additional data
        deprecated_by: Additional information about possible deprecation by
            another CPE
    """

    cpe_name: str
    cpe_name_id: str
    deprecated: bool
    last_modified: datetime
    created: datetime
    titles: List[Title] = field(default_factory=list)
    refs: List[Reference] = field(default_factory=list)
    deprecated_by: List[DeprecatedBy] = field(default_factory=list)



from typing import Iterator
import unittest

from contextlib import contextmanager
from pathlib import Path
import tomlkit

from pontos.testing import temp_directory, temp_file
from pontos.version import VersionError
from pontos.version.schemes import PEP440VersioningScheme

from pontos.version.commands._cargo import CargoVersionCommand


VERSION_EXAMPLE = """
[package]
name = "nasl-syntax"
version = "0.1.0"
edition = "2021"
license = "GPL-2.0-or-later"
"""
WORKSPACE_EXAMPLE = """
[workspace]
members = [
  "nasl-syntax",
  "nasl-interpreter",
  "nasl-cli",
  "storage",
  "redis-storage",
  "json-storage",
  "feed",
  "feed-verifier",
]
"""


class VerifyCargoUpdateCommandTestCase(unittest.TestCase):
    @contextmanager
    def __create_cargo_layout(self) -> Iterator[Path]:
        with temp_directory(change_into=True) as temporary_dir:
            workspace = temporary_dir / "Cargo.toml"
            workspace.write_text(WORKSPACE_EXAMPLE)
            members = tomlkit.parse(WORKSPACE_EXAMPLE)["workspace"]["members"]
            for cargo_workspace_member in members:
                npath = temporary_dir / f"{cargo_workspace_member}"
                npath.mkdir()
                pf = npath / "Cargo.toml"
                pf.write_text(
                    VERSION_EXAMPLE.replace(
                        "nasl-syntax", cargo_workspace_member
                    )
                )
            yield temporary_dir
        return None

    def test_update(self):
        def expected_changed_files(temporary_dir_with_cargo_toml):
            members = tomlkit.parse(WORKSPACE_EXAMPLE)["workspace"]["members"]
            return [
                (temporary_dir_with_cargo_toml / m / "Cargo.toml").resolve()
                for m in members
            ]

        with self.__create_cargo_layout() as temporary_dir_with_cargo_toml:
            cargo = CargoVersionCommand(PEP440VersioningScheme)
            previous = PEP440VersioningScheme.parse_version("0.1.0")
            new_version = PEP440VersioningScheme.parse_version("23.4.1")
            updated = cargo.update_version(new_version)
            self.assertEqual(updated.previous, previous)
            self.assertEqual(updated.new, new_version)
            self.assertEqual(
                updated.changed_files,
                expected_changed_files(temporary_dir_with_cargo_toml),
            )

    def test_update_fail(self):
        with self.__create_cargo_layout():
            cargo = CargoVersionCommand(PEP440VersioningScheme)
            previous = PEP440VersioningScheme.parse_version("0.1.0")
            new_version = PEP440VersioningScheme.parse_version("0.1.0")
            updated = cargo.update_version(new_version)
            self.assertEqual(updated.previous, previous)
            self.assertEqual(updated.new, new_version)
            self.assertEqual(
                updated.changed_files,
                [],
            )


class VerifyCargoVersionCommandTestCase(unittest.TestCase):
    def test_verify_failure(self):
        with temp_file(
            VERSION_EXAMPLE,
            name="Cargo.toml",
            change_into=True,
        ):
            version = PEP440VersioningScheme.parse_version("2.3.4")
            cargo = CargoVersionCommand(PEP440VersioningScheme)

            with self.assertRaisesRegex(
                VersionError,
                "Provided version 2.3.4 does not match the "
                "current version 0.1.0.",
            ):
                cargo.verify_version(version)

    def test_success(self):
        with temp_file(
            VERSION_EXAMPLE,
            name="Cargo.toml",
            change_into=True,
        ):
            version = PEP440VersioningScheme.parse_version("0.1.0")
            cargo = CargoVersionCommand(PEP440VersioningScheme)
            cargo.verify_version(version)



# Copyright (C) 2023 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import argparse
from typing import List, Optional

from pontos.errors import PontosError
from pontos.version.schemes import (
    VERSIONING_SCHEMES,
    VersioningScheme,
    versioning_scheme_argument_type,
)


def initialize_default_parser() -> argparse.ArgumentParser:
    """
    Returns a default argument parser containing:
    - verify
    - show
    - update
    """
    parser = argparse.ArgumentParser(
        description="Version handling utilities.",
        prog="version",
    )
    subparsers = parser.add_subparsers(
        title="subcommands",
        description="Valid subcommands",
        help="Additional help",
        dest="command",
        required=True,
    )

    verify_parser = subparsers.add_parser("verify")
    verify_parser.add_argument(
        "version",
        help="Version string to compare",
        nargs="?",
    )
    verify_parser.add_argument(
        "--versioning-scheme",
        help="Versioning scheme to use for parsing and handling version "
        f"information. Choices are {', '.join(VERSIONING_SCHEMES.keys())}. "
        "Default: %(default)s",
        default="pep440",
        type=versioning_scheme_argument_type,
    )

    show_parser = subparsers.add_parser("show")
    show_parser.add_argument(
        "--versioning-scheme",
        help="Versioning scheme to use for parsing and handling version "
        f"information. Choices are {', '.join(VERSIONING_SCHEMES.keys())}. "
        "Default: %(default)s",
        default="pep440",
        type=versioning_scheme_argument_type,
    )

    update_parser = subparsers.add_parser("update")
    update_parser.add_argument(
        "version",
        help="Version string to use",
    )
    update_parser.add_argument(
        "--versioning-scheme",
        help="Versioning scheme to use for parsing and handling version "
        f"information. Choices are {', '.join(VERSIONING_SCHEMES.keys())}. "
        "Default: %(default)s",
        default="pep440",
        type=versioning_scheme_argument_type,
    )
    update_parser.add_argument(
        "--force",
        help="Don't check if version is already set. "
        "This will override existing version information!",
        action="store_true",
    )

    next_parser = subparsers.add_parser(
        "next", help="Calculate the next release version"
    )
    next_parser.add_argument(
        "type",
        help="Next version type",
        choices=[
            "dev",
            "calendar",
            "alpha",
            "beta",
            "rc",
            "patch",
            "minor",
            "major",
        ],
    )
    next_parser.add_argument(
        "--versioning-scheme",
        help="Versioning scheme to use for parsing and handling version "
        f"information. Choices are {', '.join(VERSIONING_SCHEMES.keys())}. "
        "Default: %(default)s",
        default="pep440",
        type=versioning_scheme_argument_type,
    )
    return parser


def parse_args(args: Optional[List[str]] = None) -> argparse.Namespace:
    parser = initialize_default_parser()

    parsed_args = parser.parse_args(args)
    scheme: VersioningScheme = parsed_args.versioning_scheme

    version = getattr(parsed_args, "version", None)
    if version and version != "current":
        try:
            parsed_args.version = scheme.parse_version(parsed_args.version)
        except PontosError as e:
            parser.error(str(e))
    return parsed_args



# Copyright (C) 2022 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import unittest
from pathlib import Path
from unittest.mock import patch

from pontos.git import Git
from pontos.git.git import (
    DEFAULT_TAG_SORT_SUFFIX,
    ConfigScope,
    GitError,
    MergeStrategy,
    TagSort,
)
from pontos.git.status import Status
from pontos.testing import temp_directory, temp_git_repository


class GitTestCase(unittest.TestCase):
    @patch("pontos.git.git.exec_git")
    def test_exec(self, exec_git_mock):
        git = Git()
        git.exec("foo", "bar", "baz")

        exec_git_mock.assert_called_once_with("foo", "bar", "baz", cwd=None)

    @patch("pontos.git.git.exec_git")
    def test_clone(self, exec_git_mock):
        git = Git()
        git.clone("http://foo/foo.git", Path("/bar"))

        exec_git_mock.assert_called_once_with(
            "clone", "http://foo/foo.git", "/bar", cwd=None
        )

    @patch("pontos.git.git.exec_git")
    def test_clone_with_remote(self, exec_git_mock):
        git = Git()
        git.clone("http://foo/foo.git", Path("/bar"), remote="foo")

        exec_git_mock.assert_called_once_with(
            "clone", "-o", "foo", "http://foo/foo.git", "/bar", cwd=None
        )

    @patch("pontos.git.git.exec_git")
    def test_clone_with_branch(self, exec_git_mock):
        git = Git()
        git.clone("http://foo/foo.git", Path("/bar"), branch="foo")

        exec_git_mock.assert_called_once_with(
            "clone", "-b", "foo", "http://foo/foo.git", "/bar", cwd=None
        )

    @patch("pontos.git.git.exec_git")
    def test_clone_with_depth(self, exec_git_mock):
        git = Git()
        git.clone("http://foo/foo.git", Path("/bar"), depth=1)

        exec_git_mock.assert_called_once_with(
            "clone", "--depth", "1", "http://foo/foo.git", "/bar", cwd=None
        )

    @patch("pontos.git.git.exec_git")
    def test_init(self, exec_git_mock):
        git = Git()
        git.init()

        exec_git_mock.assert_called_once_with("init", cwd=None)

    @patch("pontos.git.git.exec_git")
    def test_init_bare(self, exec_git_mock):
        git = Git()
        git.init(bare=True)

        exec_git_mock.assert_called_once_with("init", "--bare", cwd=None)

    def test_cwd(self):
        git = Git()

        self.assertIsNone(git.cwd)

        new_cwd = Path("foo")
        git.cwd = new_cwd

        self.assertEqual(git.cwd, new_cwd.absolute())

    @patch("pontos.git.git.exec_git")
    def test_create_branch(self, exec_git_mock):
        git = Git()
        git.create_branch("foo")

        exec_git_mock.assert_called_once_with("checkout", "-b", "foo", cwd=None)

    @patch("pontos.git.git.exec_git")
    def test_create_branch_with_starting_point(self, exec_git_mock):
        git = Git()
        git.create_branch("foo", start_point="bar")

        exec_git_mock.assert_called_once_with(
            "checkout", "-b", "foo", "bar", cwd=None
        )

    @patch("pontos.git.git.exec_git")
    def test_rebase(self, exec_git_mock):
        git = Git()
        git.rebase("foo")

        exec_git_mock.assert_called_once_with("rebase", "foo", cwd=None)

    @patch("pontos.git.git.exec_git")
    def test_rebase_with_head(self, exec_git_mock):
        git = Git()
        git.rebase("foo", head="bar")

        exec_git_mock.assert_called_once_with("rebase", "foo", "bar", cwd=None)

    @patch("pontos.git.git.exec_git")
    def test_rebase_with_head_and_onto(self, exec_git_mock):
        git = Git()
        git.rebase("foo", head="bar", onto="staging")

        exec_git_mock.assert_called_once_with(
            "rebase", "--onto", "staging", "foo", "bar", cwd=None
        )

    @patch("pontos.git.git.exec_git")
    def test_rebase_with_octopus_strategy(self, exec_git_mock):
        git = Git()
        git.rebase("foo", strategy=MergeStrategy.OCTOPUS)

        exec_git_mock.assert_called_once_with(
            "rebase", "--strategy", "octopus", "foo", cwd=None
        )

    @patch("pontos.git.git.exec_git")
    def test_rebase_with_ort_ours_strategy(self, exec_git_mock):
        git = Git()
        git.rebase("foo", strategy=MergeStrategy.ORT_OURS)

        exec_git_mock.assert_called_once_with(
            "rebase", "--strategy", "ort", "-X", "ours", "foo", cwd=None
        )

    @patch("pontos.git.git.exec_git")
    def test_push(self, exec_git_mock):
        git = Git()
        git.push()

        exec_git_mock.assert_called_once_with("push", cwd=None)

    @patch("pontos.git.git.exec_git")
    def test_push_with_remote(self, exec_git_mock):
        git = Git()
        git.push(remote="foo")

        exec_git_mock.assert_called_once_with("push", "foo", cwd=None)

    @patch("pontos.git.git.exec_git")
    def test_push_with_remote_and_branch(self, exec_git_mock):
        git = Git()
        git.push(remote="foo", branch="bar")

        exec_git_mock.assert_called_once_with("push", "foo", "bar", cwd=None)

    @patch("pontos.git.git.exec_git")
    def test_push_with_follow_tags(self, exec_git_mock):
        git = Git()
        git.push(follow_tags=True)

        exec_git_mock.assert_called_once_with("push", "--follow-tags", cwd=None)

    @patch("pontos.git.git.exec_git")
    def test_push_with_follow_tags_false(self, exec_git_mock):
        git = Git()
        git.push(follow_tags=False)

        exec_git_mock.assert_called_once_with("push", cwd=None)

    @patch("pontos.git.git.exec_git")
    def test_push_with_force(self, exec_git_mock):
        git = Git()
        git.push(force=True)

        exec_git_mock.assert_called_once_with("push", "--force", cwd=None)

    @patch("pontos.git.git.exec_git")
    def test_push_with_force_false(self, exec_git_mock):
        git = Git()
        git.push(force=False)

        exec_git_mock.assert_called_once_with("push", cwd=None)

    @patch("pontos.git.git.exec_git")
    def test_config_get(self, exec_git_mock):
        git = Git()
        git.config("foo")

        exec_git_mock.assert_called_once_with("config", "foo", cwd=None)

    @patch("pontos.git.git.exec_git")
    def test_config_set(self, exec_git_mock):
        git = Git()
        git.config("foo", "bar")

        exec_git_mock.assert_called_once_with("config", "foo", "bar", cwd=None)

    @patch("pontos.git.git.exec_git")
    def test_config_get_local_scope(self, exec_git_mock):
        git = Git()
        git.config("foo", scope=ConfigScope.LOCAL)

        exec_git_mock.assert_called_once_with(
            "config", "--local", "foo", cwd=None
        )

    @patch("pontos.git.git.exec_git")
    def test_config_get_system_scope(self, exec_git_mock):
        git = Git()
        git.config("foo", scope=ConfigScope.SYSTEM)

        exec_git_mock.assert_called_once_with(
            "config", "--system", "foo", cwd=None
        )

    @patch("pontos.git.git.exec_git")
    def test_config_get_global_scope(self, exec_git_mock):
        git = Git()
        git.config("foo", scope=ConfigScope.GLOBAL)

        exec_git_mock.assert_called_once_with(
            "config", "--global", "foo", cwd=None
        )

    @patch("pontos.git.git.exec_git")
    def test_config_get_worktree_scope(self, exec_git_mock):
        git = Git()
        git.config("foo", scope=ConfigScope.WORKTREE)

        exec_git_mock.assert_called_once_with(
            "config", "--worktree", "foo", cwd=None
        )

    @patch("pontos.git.git.exec_git")
    def test_cherry_pick(self, exec_git_mock):
        git = Git()
        git.cherry_pick(["foo", "bar"])

        exec_git_mock.assert_called_once_with(
            "cherry-pick", "foo", "bar", cwd=None
        )

    @patch("pontos.git.git.exec_git")
    def test_cherry_pick_single_commit(self, exec_git_mock):
        git = Git()
        git.cherry_pick("foo")

        exec_git_mock.assert_called_once_with("cherry-pick", "foo", cwd=None)

    @patch("pontos.git.git.exec_git")
    def test_list_tags(self, exec_git_mock):
        exec_git_mock.return_value = "v1.0\nv2.0\nv2.1\n"
        git = Git()
        tags = git.list_tags()

        exec_git_mock.assert_called_once_with("tag", "-l", cwd=None)

        self.assertEqual(len(tags), 3)
        self.assertEqual(tags[0], "v1.0")
        self.assertEqual(tags[1], "v2.0")
        self.assertEqual(tags[2], "v2.1")

    @patch("pontos.git.git.exec_git")
    def test_list_tags_with_version_sort(self, exec_git_mock):
        exec_git_mock.return_value = "v1.0\nv2.0\nv2.1\n"
        git = Git()
        tags = git.list_tags(sort=TagSort.VERSION)

        exec_git_mock.assert_called_once_with(
            "tag",
            "-l",
            "--sort=version:refname",
            cwd=None,
        )

        self.assertEqual(len(tags), 3)
        self.assertEqual(tags[0], "v1.0")
        self.assertEqual(tags[1], "v2.0")
        self.assertEqual(tags[2], "v2.1")

    @patch("pontos.git.git.exec_git")
    def test_list_tags_with_version_suffix_sort(self, exec_git_mock):
        exec_git_mock.return_value = "v1.0\nv2.0\nv2.1\n"
        git = Git()
        tags = git.list_tags(
            sort=TagSort.VERSION, sort_suffix=DEFAULT_TAG_SORT_SUFFIX
        )

        exec_git_mock.assert_called_once_with(
            "-c",
            "versionsort.suffix=-alpha",
            "-c",
            "versionsort.suffix=a",
            "-c",
            "versionsort.suffix=-beta",
            "-c",
            "versionsort.suffix=b",
            "-c",
            "versionsort.suffix=-rc",
            "-c",
            "versionsort.suffix=rc",
            "tag",
            "-l",
            "--sort=version:refname",
            cwd=None,
        )

        self.assertEqual(len(tags), 3)
        self.assertEqual(tags[0], "v1.0")
        self.assertEqual(tags[1], "v2.0")
        self.assertEqual(tags[2], "v2.1")

    @patch("pontos.git.git.exec_git")
    def test_list_tags_with_tag_name(self, exec_git_mock):
        exec_git_mock.return_value = "v2.0\nv2.1\n"
        git = Git()
        tags = git.list_tags(tag_name="v2.*")

        exec_git_mock.assert_called_once_with("tag", "-l", "v2.*", cwd=None)

        self.assertEqual(len(tags), 2)
        self.assertEqual(tags[0], "v2.0")
        self.assertEqual(tags[1], "v2.1")

    @patch("pontos.git.git.exec_git")
    def test_add_single_file(self, exec_git_mock):
        git = Git()
        git.add("foo")

        exec_git_mock.assert_called_once_with("add", "foo", cwd=None)

    @patch("pontos.git.git.exec_git")
    def test_add(self, exec_git_mock):
        git = Git()
        git.add(["foo", "bar"])

        exec_git_mock.assert_called_once_with("add", "foo", "bar", cwd=None)

    @patch("pontos.git.git.exec_git")
    def test_commit(self, exec_git_mock):
        git = Git()
        git.commit("Add foo")

        exec_git_mock.assert_called_once_with(
            "commit", "-m", "Add foo", cwd=None
        )

    @patch("pontos.git.git.exec_git")
    def test_commit_with_signing_key(self, exec_git_mock):
        git = Git()
        git.commit(
            "Add foo",
            gpg_signing_key="8AE4BE429B60A59B311C2E739823FAA60ED1E580",
        )

        exec_git_mock.assert_called_once_with(
            "commit",
            "-S8AE4BE429B60A59B311C2E739823FAA60ED1E580",
            "-m",
            "Add foo",
            cwd=None,
        )

    @patch("pontos.git.git.exec_git")
    def test_commit_without_verify(self, exec_git_mock):
        git = Git()
        git.commit("Add foo", verify=False)

        exec_git_mock.assert_called_once_with(
            "commit", "--no-verify", "-m", "Add foo", cwd=None
        )

    @patch("pontos.git.git.exec_git")
    def test_tag(self, exec_git_mock):
        git = Git()
        git.tag(tag="test")

        exec_git_mock.assert_called_once_with("tag", "test", cwd=None)

    @patch("pontos.git.git.exec_git")
    def test_tag_with_gpg_key(self, exec_git_mock):
        git = Git()
        git.tag("test", gpg_key_id="0x123")

        exec_git_mock.assert_called_once_with(
            "tag", "-u", "0x123", "test", cwd=None
        )

    @patch("pontos.git.git.exec_git")
    def test_tag_with_message(self, exec_git_mock):
        git = Git()
        git.tag("test", message="Tag for 123 release")

        exec_git_mock.assert_called_once_with(
            "tag", "-m", "Tag for 123 release", "test", cwd=None
        )

    @patch("pontos.git.git.exec_git")
    def test_tag_with_force(self, exec_git_mock):
        git = Git()
        git.tag("test", force=True)

        exec_git_mock.assert_called_once_with(
            "tag", "--force", "test", cwd=None
        )

    @patch("pontos.git.git.exec_git")
    def test_fetch(self, exec_git_mock):
        git = Git()
        git.fetch()

        exec_git_mock.assert_called_once_with("fetch", cwd=None)

    @patch("pontos.git.git.exec_git")
    def test_fetch_with_remote(self, exec_git_mock):
        git = Git()
        git.fetch("foo")

        exec_git_mock.assert_called_once_with("fetch", "foo", cwd=None)

    @patch("pontos.git.git.exec_git")
    def test_fetch_with_remote_and_refspec(self, exec_git_mock):
        git = Git()
        git.fetch("foo", "my-branch")

        exec_git_mock.assert_called_once_with(
            "fetch", "foo", "my-branch", cwd=None
        )

    @patch("pontos.git.git.exec_git")
    def test_fetch_with_verbose(self, exec_git_mock):
        git = Git()
        git.fetch(verbose=True)

        exec_git_mock.assert_called_once_with("fetch", "-v", cwd=None)

    @patch("pontos.git.git.exec_git")
    def test_add_remote(self, exec_git_mock):
        git = Git()
        git.add_remote("foo", "https://foo.bar/foo.git")

        exec_git_mock.assert_called_once_with(
            "remote", "add", "foo", "https://foo.bar/foo.git", cwd=None
        )

    @patch("pontos.git.git.exec_git")
    def test_checkout(self, exec_git_mock):
        git = Git()
        git.checkout("foo")

        exec_git_mock.assert_called_once_with("checkout", "foo", cwd=None)

    @patch("pontos.git.git.exec_git")
    def test_checkout_with_start_point(self, exec_git_mock):
        git = Git()
        git.checkout("foo", start_point="bar")

        exec_git_mock.assert_called_once_with(
            "checkout", "-b", "foo", "bar", cwd=None
        )

    @patch("pontos.git.git.exec_git")
    def test_remote_url(self, exec_git_mock):
        url = "git@github.com:foo/foo.git"
        exec_git_mock.return_value = url

        git = Git()
        remote = git.remote_url("foo")

        exec_git_mock.assert_called_once_with(
            "remote", "get-url", "foo", cwd=None
        )

        self.assertEqual(remote, url)

    @patch("pontos.git.git.exec_git")
    def test_remote_url_with_default(self, exec_git_mock):
        url = "git@github.com:foo/foo.git"
        exec_git_mock.return_value = url

        git = Git()
        remote = git.remote_url()

        exec_git_mock.assert_called_once_with(
            "remote", "get-url", "origin", cwd=None
        )

        self.assertEqual(remote, url)

    @patch("pontos.git.git.exec_git")
    def test_log(self, exec_git_mock):
        # pylint: disable=line-too-long
        exec_git_mock.return_value = """commit 68c6c3785bbb049df63dc51f8b5b709eb19f8517
Author: Björn Ricks <bjoern.ricks@greenbone.net>
Date:   Wed Apr 8 15:16:05 2020 +0200

    Add a draft for a README.md document

commit 464f24d43d7293091b168c6b37ee37978a650958
Author: Björn Ricks <bjoern.ricks@greenbone.net>
Date:   Wed Apr 8 14:28:53 2020 +0200

    Initial commit
"""

        git = Git()
        logs = git.log()

        exec_git_mock.assert_called_once_with("log", cwd=None)

        self.assertEqual(
            logs[0], "commit 68c6c3785bbb049df63dc51f8b5b709eb19f8517"
        )
        self.assertEqual(
            logs[6], "commit 464f24d43d7293091b168c6b37ee37978a650958"
        )

    @patch("pontos.git.git.exec_git")
    def test_log_with_oneline(self, exec_git_mock):
        exec_git_mock.return_value = """50f9963 Add CircleCI config for pontos
9a8feaa Rename to pontos only
047cfae Update README for installation and development
e6ea80d Update README
68c6c37 Add a draft for a README.md document
464f24d Initial commit"""

        git = Git()
        logs = git.log(oneline=True)

        exec_git_mock.assert_called_once_with("log", "--oneline", cwd=None)

        self.assertEqual(logs[0], "50f9963 Add CircleCI config for pontos")
        self.assertEqual(logs[5], "464f24d Initial commit")

    @patch("pontos.git.git.exec_git")
    def test_rev_list(self, exec_git_mock):
        git = Git()
        git.rev_list("foo", "bar", "baz", max_parents=123, abbrev_commit=True)

        exec_git_mock.assert_called_once_with(
            "rev-list",
            "--max-parents=123",
            "--abbrev-commit",
            "foo",
            "bar",
            "baz",
            cwd=None,
        )

    @patch("pontos.git.git.exec_git")
    def test_move(self, exec_git_mock):
        git = Git()
        git.move("foo", "bar")

        exec_git_mock.assert_called_once_with(
            "mv",
            "foo",
            "bar",
            cwd=None,
        )

    @patch("pontos.git.git.exec_git")
    def test_remove(self, exec_git_mock):
        git = Git()
        git.remove("foo")

        exec_git_mock.assert_called_once_with(
            "rm",
            "foo",
            cwd=None,
        )

    @patch("pontos.git.git.exec_git")
    def test_status(self, exec_git_mock):
        git = Git()
        git.status()

        exec_git_mock.assert_called_once_with(
            "status",
            "-z",
            "--ignore-submodules",
            "--untracked-files=no",
            cwd=None,
        )

    @patch("pontos.git.git.exec_git")
    def test_status_with_files(self, exec_git_mock):
        git = Git()
        git.status(["foo", "bar", "baz"])

        exec_git_mock.assert_called_once_with(
            "status",
            "-z",
            "--ignore-submodules",
            "--untracked-files=no",
            "--",
            "foo",
            "bar",
            "baz",
            cwd=None,
        )


class GitExtendedTestCase(unittest.TestCase):
    def test_semantic_list_tags(self):
        with temp_git_repository() as tmp_git:
            tags = [
                "v0.6.5-alpha3",
                "v0.6.5-beta1",
                "v0.6.5-alpha2",
                "v0.6.5-rc1",
                "v0.6.5-alpha1",
                "v0.6.5",
                "v0.6.4",
                "v0.6.3-alpha1",
                "v1.0.0",
            ]
            git = Git(tmp_git)
            git.config("commit.gpgSign", "false", scope=ConfigScope.LOCAL)
            git.config("tag.gpgSign", "false", scope=ConfigScope.LOCAL)
            git.config("tag.sort", "refname", scope=ConfigScope.LOCAL)

            tmp_file = tmp_git / "foo.txt"
            tmp_file.touch()

            git.add(tmp_file)

            git.commit("some commit")

            for tag in tags:
                git.tag(tag)

            tags = git.list_tags()
            self.assertEqual(
                tags,
                [
                    "v0.6.3-alpha1",
                    "v0.6.4",
                    "v0.6.5",
                    "v0.6.5-alpha1",
                    "v0.6.5-alpha2",
                    "v0.6.5-alpha3",
                    "v0.6.5-beta1",
                    "v0.6.5-rc1",
                    "v1.0.0",
                ],
            )

            tags = git.list_tags(sort=TagSort.VERSION)
            self.assertEqual(
                tags,
                [
                    "v0.6.3-alpha1",
                    "v0.6.4",
                    "v0.6.5",
                    "v0.6.5-alpha1",
                    "v0.6.5-alpha2",
                    "v0.6.5-alpha3",
                    "v0.6.5-beta1",
                    "v0.6.5-rc1",
                    "v1.0.0",
                ],
            )

            tags = git.list_tags(sort=TagSort.VERSION, tag_name="v0.6*")
            self.assertEqual(
                tags,
                [
                    "v0.6.3-alpha1",
                    "v0.6.4",
                    "v0.6.5",
                    "v0.6.5-alpha1",
                    "v0.6.5-alpha2",
                    "v0.6.5-alpha3",
                    "v0.6.5-beta1",
                    "v0.6.5-rc1",
                ],
            )

            tags = git.list_tags(sort=TagSort.VERSION, tag_name="v0.6.5*")
            self.assertEqual(
                tags,
                [
                    "v0.6.5",
                    "v0.6.5-alpha1",
                    "v0.6.5-alpha2",
                    "v0.6.5-alpha3",
                    "v0.6.5-beta1",
                    "v0.6.5-rc1",
                ],
            )

            tags = git.list_tags(tag_name="v0.6.5*")
            self.assertEqual(
                tags,
                [
                    "v0.6.5",
                    "v0.6.5-alpha1",
                    "v0.6.5-alpha2",
                    "v0.6.5-alpha3",
                    "v0.6.5-beta1",
                    "v0.6.5-rc1",
                ],
            )

            tags = git.list_tags(
                sort=TagSort.VERSION, sort_suffix=DEFAULT_TAG_SORT_SUFFIX
            )
            self.assertEqual(
                tags,
                [
                    "v0.6.3-alpha1",
                    "v0.6.4",
                    "v0.6.5-alpha1",
                    "v0.6.5-alpha2",
                    "v0.6.5-alpha3",
                    "v0.6.5-beta1",
                    "v0.6.5-rc1",
                    "v0.6.5",
                    "v1.0.0",
                ],
            )

    def test_pep440_list_tags(self):
        with temp_git_repository() as tmp_git:
            tags = [
                "v0.6.5a3",
                "v0.6.5b1",
                "v0.6.5a2",
                "v0.6.5rc1",
                "v0.6.5a1",
                "v0.6.5",
                "v0.6.4",
                "v0.6.3a1",
                "v1.0.0",
            ]
            git = Git(tmp_git)
            git.config("commit.gpgSign", "false", scope=ConfigScope.LOCAL)
            git.config("tag.gpgSign", "false", scope=ConfigScope.LOCAL)
            git.config("tag.sort", "refname", scope=ConfigScope.LOCAL)

            tmp_file = tmp_git / "foo.txt"
            tmp_file.touch()

            git.add(tmp_file)

            git.commit("some commit")

            for tag in tags:
                git.tag(tag)

            tags = git.list_tags()
            self.assertEqual(
                tags,
                [
                    "v0.6.3a1",
                    "v0.6.4",
                    "v0.6.5",
                    "v0.6.5a1",
                    "v0.6.5a2",
                    "v0.6.5a3",
                    "v0.6.5b1",
                    "v0.6.5rc1",
                    "v1.0.0",
                ],
            )

            tags = git.list_tags(sort=TagSort.VERSION)
            self.assertEqual(
                tags,
                [
                    "v0.6.3a1",
                    "v0.6.4",
                    "v0.6.5",
                    "v0.6.5a1",
                    "v0.6.5a2",
                    "v0.6.5a3",
                    "v0.6.5b1",
                    "v0.6.5rc1",
                    "v1.0.0",
                ],
            )

            tags = git.list_tags(
                sort=TagSort.VERSION, sort_suffix=DEFAULT_TAG_SORT_SUFFIX
            )
            self.assertEqual(
                tags,
                [
                    "v0.6.3a1",
                    "v0.6.4",
                    "v0.6.5a1",
                    "v0.6.5a2",
                    "v0.6.5a3",
                    "v0.6.5b1",
                    "v0.6.5rc1",
                    "v0.6.5",
                    "v1.0.0",
                ],
            )

    def test_git_status(self):
        with temp_git_repository() as tmp_git:
            tracked_file = tmp_git / "foo.json"
            tracked_file.write_text("sed diam nonumy eirmod", encoding="utf8")
            changed_file = tmp_git / "bar.json"
            changed_file.touch()
            staged_changed_file = tmp_git / "ipsum.json"
            staged_changed_file.write_text("tempor invidunt ut labore")
            removed_file = tmp_git / "lorem.json"
            removed_file.write_text(
                "consetetur sadipscing elitr", encoding="utf8"
            )
            renamed_file = tmp_git / "foo.md"
            renamed_file.write_text(
                "et dolore magna aliquyam erat", encoding="utf8"
            )

            git = Git(tmp_git)
            git.config("commit.gpgSign", "false", scope=ConfigScope.LOCAL)
            git.add(
                [
                    tracked_file,
                    changed_file,
                    staged_changed_file,
                    removed_file,
                    renamed_file,
                ]
            )
            git.commit("Some commit")

            changed_file.write_text("Lorem Ipsum", encoding="utf8")
            staged_changed_file.write_text("Lorem Ipsum", encoding="utf8")

            added_file = tmp_git / "foo.txt"
            added_file.touch()

            added_modified_file = tmp_git / "ipsum.txt"
            added_modified_file.touch()

            git.add([added_file, staged_changed_file, added_modified_file])

            staged_changed_file.write_text("Dolor sit", encoding="utf8")

            added_modified_file.write_text("Lorem Ipsum", encoding="utf8")

            git.move(renamed_file, "foo.rst")
            git.remove(removed_file)

            untracked_file = tmp_git / "bar.txt"
            untracked_file.touch()

            it = git.status()

            status = next(it)
            self.assertEqual(status.index, Status.UNMODIFIED)
            self.assertEqual(status.working_tree, Status.MODIFIED)
            self.assertEqual(status.path, Path("bar.json"))

            status = next(it)
            self.assertEqual(status.index, Status.RENAMED)
            self.assertEqual(status.working_tree, Status.UNMODIFIED)
            self.assertEqual(status.path, Path("foo.rst"))
            self.assertEqual(status.old_path, Path("foo.md"))

            status = next(it)
            self.assertEqual(status.index, Status.ADDED)
            self.assertEqual(status.working_tree, Status.UNMODIFIED)
            self.assertEqual(status.path, Path("foo.txt"))

            status = next(it)
            self.assertEqual(status.index, Status.MODIFIED)
            self.assertEqual(status.working_tree, Status.MODIFIED)
            self.assertEqual(status.path, Path("ipsum.json"))

            status = next(it)
            self.assertEqual(status.index, Status.ADDED)
            self.assertEqual(status.working_tree, Status.MODIFIED)
            self.assertEqual(status.path, Path("ipsum.txt"))

            status = next(it)
            self.assertEqual(status.index, Status.DELETED)
            self.assertEqual(status.working_tree, Status.UNMODIFIED)
            self.assertEqual(status.path, Path("lorem.json"))

            with self.assertRaises(StopIteration):
                next(it)

    def test_git_error(self):
        with temp_directory(change_into=True), self.assertRaises(
            GitError
        ) as cm:
            Git().log()

        self.assertEqual(cm.exception.returncode, 128)
        self.assertEqual(
            cm.exception.stderr,
            "fatal: not a git repository (or any of the parent directories): "
            ".git\n",
        )
        self.assertEqual(cm.exception.stdout, "")
        self.assertEqual(cm.exception.cmd, ["git", "log"])



# Copyright (C) 2023 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from typing import Iterable, Tuple, Type

from ._cargo import CargoVersionCommand
from ._cmake import CMakeVersionCommand
from ._command import VersionCommand
from ._go import GoVersionCommand
from ._java import JavaVersionCommand
from ._javascript import JavaScriptVersionCommand
from ._python import PythonVersionCommand

__all__ = (
    "VersionCommand",
    "CMakeVersionCommand",
    "GoVersionCommand",
    "JavaScriptVersionCommand",
    "JavaVersionCommand",
    "PythonVersionCommand",
    "CargoVersionCommand",
    "get_commands",
)

__COMMANDS: Tuple[Type[VersionCommand]] = (
    CMakeVersionCommand,
    GoVersionCommand,
    JavaVersionCommand,
    JavaScriptVersionCommand,
    PythonVersionCommand,
    CargoVersionCommand,
)


def get_commands() -> Iterable[Type[VersionCommand]]:
    """
    Returns the available VersionCommands
    """
    return __COMMANDS



# Copyright (C) 2023 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import unittest
from contextlib import redirect_stderr, redirect_stdout
from datetime import datetime
from io import StringIO

from pontos.testing import temp_directory
from pontos.version.main import VersionExitCode, main


class MainTestCase(unittest.TestCase):
    def test_no_command(self):
        with self.assertRaises(SystemExit), redirect_stderr(StringIO()):
            main([])

    def test_no_project(self):
        with temp_directory(change_into=True), self.assertRaises(
            SystemExit
        ) as cm, redirect_stderr(StringIO()):
            main(["show"])

        self.assertEqual(cm.exception.code, VersionExitCode.NO_PROJECT)

    def test_update_success(self):
        with temp_directory(change_into=True) as temp_dir, redirect_stdout(
            StringIO()
        ) as out, self.assertRaises(SystemExit) as cm:
            version_file = temp_dir / "package.json"
            version_file.write_text(
                '{"name": "foo", "version": "1.2.3"}',
                encoding="utf8",
            )
            main(["update", "2.0.0"])

        self.assertEqual(
            out.getvalue(), "Updated version from 1.2.3 to 2.0.0.\n"
        )

        self.assertEqual(cm.exception.code, VersionExitCode.SUCCESS)

    def test_update_failure(self):
        with temp_directory(change_into=True) as temp_dir, redirect_stderr(
            StringIO()
        ), self.assertRaises(SystemExit) as cm:
            version_file = temp_dir / "package.json"
            version_file.write_text(
                "{}",
                encoding="utf8",
            )
            main(["update", "2.0.0"])

        self.assertEqual(cm.exception.code, VersionExitCode.UPDATE_ERROR)

    def test_update_already_up_to_date(self):
        with temp_directory(change_into=True) as temp_dir, redirect_stdout(
            StringIO()
        ) as out, self.assertRaises(SystemExit) as cm:
            version_file = temp_dir / "package.json"
            version_file.write_text(
                '{"name": "foo", "version": "1.2.3"}',
                encoding="utf8",
            )
            main(["update", "1.2.3"])

        self.assertEqual(out.getvalue(), "Version is already up-to-date.\n")

        self.assertEqual(cm.exception.code, VersionExitCode.SUCCESS)

    def test_show(self):
        with temp_directory(change_into=True) as temp_dir, redirect_stdout(
            StringIO()
        ) as out, self.assertRaises(SystemExit) as cm:
            version_file = temp_dir / "package.json"
            version_file.write_text(
                '{"name": "foo", "version": "1.2.3"}',
                encoding="utf8",
            )
            main(["show"])

        self.assertEqual(out.getvalue(), "1.2.3\n")

        self.assertEqual(cm.exception.code, VersionExitCode.SUCCESS)

    def test_show_error(self):
        with temp_directory(change_into=True) as temp_dir, redirect_stderr(
            StringIO()
        ) as out, self.assertRaises(SystemExit) as cm:
            version_file = temp_dir / "package.json"
            version_file.write_text(
                '{"name": "foo", "version": "abc"}',
                encoding="utf8",
            )
            main(["show"])

        self.assertEqual(out.getvalue(), "Invalid version: 'abc'\n")

        self.assertEqual(
            cm.exception.code, VersionExitCode.CURRENT_VERSION_ERROR
        )

    def test_verify_failure(self):
        with temp_directory(change_into=True) as temp_dir, redirect_stderr(
            StringIO()
        ) as out, self.assertRaises(SystemExit) as cm:
            version_file = temp_dir / "package.json"
            version_file.write_text(
                '{"name": "foo", "version": "1.2.3"}',
                encoding="utf8",
            )
            main(["verify", "1.2.4"])

        self.assertEqual(
            out.getvalue(),
            "Provided version 1.2.4 does not match the current version "
            f"1.2.3 in {version_file.resolve()}.\n",
        )

        self.assertEqual(cm.exception.code, VersionExitCode.VERIFY_ERROR)

    def test_verify_success(self):
        with temp_directory(change_into=True) as temp_dir, self.assertRaises(
            SystemExit
        ) as cm:
            version_file = temp_dir / "package.json"
            version_file.write_text(
                '{"name": "foo", "version": "1.2.3"}',
                encoding="utf8",
            )
            main(["verify", "1.2.3"])

        self.assertEqual(cm.exception.code, VersionExitCode.SUCCESS)

    def test_verify_current(self):
        with temp_directory(change_into=True) as temp_dir, self.assertRaises(
            SystemExit
        ) as cm:
            version_file = temp_dir / "package.json"
            version_file.write_text(
                '{"name": "foo", "version": "1.2.3"}',
                encoding="utf8",
            )
            main(["verify", "current"])

        self.assertEqual(cm.exception.code, VersionExitCode.SUCCESS)

    def test_next_invalid_current(self):
        with redirect_stderr(StringIO()), temp_directory(
            change_into=True
        ) as temp_dir, self.assertRaises(SystemExit) as cm:
            version_file = temp_dir / "package.json"
            version_file.write_text(
                '{"name": "foo", "version": "1.2.tt"}',
                encoding="utf8",
            )
            main(["next", "dev", "--versioning-scheme", "pep440"])

        self.assertEqual(
            cm.exception.code, VersionExitCode.CURRENT_VERSION_ERROR
        )

    def test_next_dev(self):
        with temp_directory(change_into=True) as temp_dir, redirect_stdout(
            StringIO()
        ) as out, self.assertRaises(SystemExit) as cm:
            version_file = temp_dir / "package.json"
            version_file.write_text(
                '{"name": "foo", "version": "1.2.3"}',
                encoding="utf8",
            )
            main(["next", "dev", "--versioning-scheme", "pep440"])

        self.assertEqual(cm.exception.code, VersionExitCode.SUCCESS)
        self.assertEqual(out.getvalue(), "1.2.4.dev1\n")

    def test_next_patch(self):
        with temp_directory(change_into=True) as temp_dir, redirect_stdout(
            StringIO()
        ) as out, self.assertRaises(SystemExit) as cm:
            version_file = temp_dir / "package.json"
            version_file.write_text(
                '{"name": "foo", "version": "1.2.3"}',
                encoding="utf8",
            )
            main(["next", "patch", "--versioning-scheme", "pep440"])

        self.assertEqual(cm.exception.code, VersionExitCode.SUCCESS)
        self.assertEqual(out.getvalue(), "1.2.4\n")

    def test_next_minor(self):
        with temp_directory(change_into=True) as temp_dir, redirect_stdout(
            StringIO()
        ) as out, self.assertRaises(SystemExit) as cm:
            version_file = temp_dir / "package.json"
            version_file.write_text(
                '{"name": "foo", "version": "1.2.3"}',
                encoding="utf8",
            )
            main(["next", "minor", "--versioning-scheme", "pep440"])

        self.assertEqual(cm.exception.code, VersionExitCode.SUCCESS)
        self.assertEqual(out.getvalue(), "1.3.0\n")

    def test_next_major(self):
        with temp_directory(change_into=True) as temp_dir, redirect_stdout(
            StringIO()
        ) as out, self.assertRaises(SystemExit) as cm:
            version_file = temp_dir / "package.json"
            version_file.write_text(
                '{"name": "foo", "version": "1.2.3"}',
                encoding="utf8",
            )
            main(["next", "major", "--versioning-scheme", "pep440"])

        self.assertEqual(cm.exception.code, VersionExitCode.SUCCESS)
        self.assertEqual(out.getvalue(), "2.0.0\n")

    def test_next_alpha(self):
        with temp_directory(change_into=True) as temp_dir, redirect_stdout(
            StringIO()
        ) as out, self.assertRaises(SystemExit) as cm:
            version_file = temp_dir / "package.json"
            version_file.write_text(
                '{"name": "foo", "version": "1.2.3"}',
                encoding="utf8",
            )
            main(["next", "alpha", "--versioning-scheme", "semver"])

        self.assertEqual(cm.exception.code, VersionExitCode.SUCCESS)
        self.assertEqual(out.getvalue(), "1.2.4-alpha1\n")

    def test_next_beta(self):
        with temp_directory(change_into=True) as temp_dir, redirect_stdout(
            StringIO()
        ) as out, self.assertRaises(SystemExit) as cm:
            version_file = temp_dir / "package.json"
            version_file.write_text(
                '{"name": "foo", "version": "1.2.3"}',
                encoding="utf8",
            )
            main(["next", "beta", "--versioning-scheme", "semver"])

        self.assertEqual(cm.exception.code, VersionExitCode.SUCCESS)
        self.assertEqual(out.getvalue(), "1.2.4-beta1\n")

    def test_next_rc(self):
        with temp_directory(change_into=True) as temp_dir, redirect_stdout(
            StringIO()
        ) as out, self.assertRaises(SystemExit) as cm:
            version_file = temp_dir / "package.json"
            version_file.write_text(
                '{"name": "foo", "version": "1.2.3"}',
                encoding="utf8",
            )
            main(["next", "rc", "--versioning-scheme", "semver"])

        self.assertEqual(cm.exception.code, VersionExitCode.SUCCESS)
        self.assertEqual(out.getvalue(), "1.2.4-rc1\n")

    def test_next_calendar(self):
        today = datetime.today()
        with temp_directory(change_into=True) as temp_dir, redirect_stdout(
            StringIO()
        ) as out, self.assertRaises(SystemExit) as cm:
            version_file = temp_dir / "package.json"
            version_file.write_text(
                '{"name": "foo", "version": "1.2.3"}',
                encoding="utf8",
            )
            main(["next", "calendar", "--versioning-scheme", "pep440"])

        self.assertEqual(cm.exception.code, VersionExitCode.SUCCESS)
        self.assertEqual(
            out.getvalue(), f"{today.year % 100}.{today.month}.0\n"
        )



# Copyright (C) 2022 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import unittest
from pathlib import Path
from unittest.mock import patch

from pontos.github.actions.env import GitHubEnvironment


class GitHubEnvironmentTestCase(unittest.TestCase):
    @patch.dict("os.environ", {"GITHUB_WORKSPACE": "/foo/bar"}, clear=True)
    def test_workspace(self):
        env = GitHubEnvironment()
        self.assertEqual(env.workspace, Path("/foo/bar"))

    @patch.dict("os.environ", {"GITHUB_REPOSITORY": "foo/bar"}, clear=True)
    def test_repository(self):
        env = GitHubEnvironment()
        self.assertEqual(env.repository, "foo/bar")

    @patch.dict("os.environ", {"GITHUB_SHA": "123456"}, clear=True)
    def test_sha(self):
        env = GitHubEnvironment()
        self.assertEqual(env.sha, "123456")

    @patch.dict("os.environ", {"GITHUB_REF": "ref/branches/main"}, clear=True)
    def test_ref(self):
        env = GitHubEnvironment()
        self.assertEqual(env.ref, "ref/branches/main")

    @patch.dict("os.environ", {"GITHUB_REF_NAME": "main"}, clear=True)
    def test_ref_name(self):
        env = GitHubEnvironment()
        self.assertEqual(env.ref_name, "main")

    @patch.dict("os.environ", {"GITHUB_EVENT_PATH": "/foo/bar"}, clear=True)
    def test_event_path(self):
        env = GitHubEnvironment()
        self.assertEqual(env.event_path, Path("/foo/bar"))

    @patch.dict("os.environ", {"GITHUB_HEAD_REF": "foo"}, clear=True)
    def test_head_ref(self):
        env = GitHubEnvironment()
        self.assertEqual(env.head_ref, "foo")

    @patch.dict("os.environ", {"GITHUB_BASE_REF": "main"}, clear=True)
    def test_base_ref(self):
        env = GitHubEnvironment()
        self.assertEqual(env.base_ref, "main")

    @patch.dict(
        "os.environ", {"GITHUB_API_URL": "https://api.github.com"}, clear=True
    )
    def test_api_url(self):
        env = GitHubEnvironment()
        self.assertEqual(env.api_url, "https://api.github.com")

    @patch.dict("os.environ", {"GITHUB_ACTOR": "greenbonebot"}, clear=True)
    def test_actor(self):
        env = GitHubEnvironment()
        self.assertEqual(env.actor, "greenbonebot")

    @patch.dict("os.environ", {"GITHUB_RUN_ID": "12345"}, clear=True)
    def test_run_id(self):
        env = GitHubEnvironment()
        self.assertEqual(env.run_id, "12345")

    @patch.dict("os.environ", {"GITHUB_ACTION": "54321"}, clear=True)
    def test_action_id(self):
        env = GitHubEnvironment()
        self.assertEqual(env.action_id, "54321")

    @patch.dict("os.environ", {"RUNNER_DEBUG": "1"}, clear=True)
    def test_is_debug_enabled(self):
        env = GitHubEnvironment()
        self.assertTrue(env.is_debug)

    @patch.dict("os.environ", {"RUNNER_DEBUG": ""}, clear=True)
    def test_is_debug_disabled(self):
        env = GitHubEnvironment()
        self.assertFalse(env.is_debug)



# Copyright (C) 2022 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
This script prints out all artifacts of a given repository
"""

from argparse import ArgumentParser, Namespace

from rich.console import Console
from rich.table import Table

from pontos.github.api import GitHubAsyncRESTApi


def add_script_arguments(parser: ArgumentParser) -> None:
    parser.add_argument("repository")


async def github_script(api: GitHubAsyncRESTApi, args: Namespace) -> int:
    table = Table()
    table.add_column("Name")
    table.add_column("ID")
    table.add_column("URL")
    table.add_column("Updated")
    table.add_column("Expired")
    table.add_column("Size (in KB)", justify="right")

    count = 0
    async for artifact in api.artifacts.get_all(args.repository):
        table.add_row(
            artifact.name,
            str(artifact.id),
            f"[link={artifact.archive_download_url}]"
            f"{artifact.archive_download_url}[/link]",
            str(artifact.updated_at),
            str(artifact.expired),
            f"{artifact.size_in_bytes / 1024:.2f}",
        )
        count += 1

    console = Console()
    console.print(table)

    print(f"{count} artifacts.")
    return 0



# Copyright (C) 2022 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from dataclasses import dataclass, field
from datetime import datetime
from enum import Enum
from typing import List, Optional

from pontos.github.models.base import GitHubModel, User

__all__ = (
    "CodeOfConduct",
    "GitIgnoreTemplate",
    "InvitationRole",
    "License",
    "LicenseType",
    "MemberFilter",
    "MemberRole",
    "MergeCommitMessage",
    "MergeCommitTitle",
    "Organization",
    "Repository",
    "RepositoryPermissions",
    "RepositoryType",
    "SecurityAndAnalysis",
    "SecurityAndAnalysisStatus",
    "SecurityAndAnalysisType",
    "SquashMergeCommitMessage",
    "SquashMergeCommitTitle",
)


class MergeCommitTitle(Enum):
    """
    Merge commit title

    Attributes:
        PR_TITLE: Use pull request title
        MERGE_MESSAGE: Use provided merge commit message
    """

    PR_TITLE = "PR_TITLE"
    MERGE_MESSAGE = "MERGE_MESSAGE"


class MergeCommitMessage(Enum):
    """
    Merge commit message setting

    Attributes:
        PR_BODY: Use pull request body
        PR_TITLE: Use pull request title
        BLANK: Leave it blank
    """

    PR_BODY = "PR_BODY"
    PR_TITLE = "PR_TITLE"
    BLANK = "BLANK"


class SquashMergeCommitTitle(Enum):
    """
    Squash merge commit title

    Attributes:
        PR_TITLE: Use pull request title
        COMMIT_OR_PR_TITLE: Use pull request or commit title
    """

    PR_TITLE = "PR_TITLE"
    COMMIT_OR_PR_TITLE = "COMMIT_OR_PR_TITLE"


class SquashMergeCommitMessage(Enum):
    """
    Squash merge commit message setting

    Attributes:
        PR_BODY: Use pull request body
        COMMIT_MESSAGES: Use commit messages
        BLANK: Leave it blank
    """

    PR_BODY = "PR_BODY"
    COMMIT_MESSAGES = "COMMIT_MESSAGES"
    BLANK = "BLANK"


class RepositoryType(Enum):
    """
    A repository type

    Attributes:
        ALL: All repository types
        PUBLIC: Public repository
        PRIVATE: Private repository
        FORKS: Forked repository
        SOURCES:
        MEMBER:
        INTERNAL:
    """

    ALL = "all"
    PUBLIC = "public"
    PRIVATE = "private"
    FORKS = "forks"
    SOURCES = "sources"
    MEMBER = "member"
    INTERNAL = "internal"


@dataclass
class License(GitHubModel):
    """
    Software License

    Attributes:
        key: Key of the license
        name: Name of the license
        node_id: Node ID of the license
        url: URL to the license
        spdx_id: SPDX ID of the license
        html_url: URL to the web page of the license
    """

    key: str
    name: str
    node_id: str
    url: Optional[str] = None
    spdx_id: Optional[str] = None
    html_url: Optional[str] = None


@dataclass
class RepositoryPermissions(GitHubModel):
    """
    GitHub repository permissions

    Attributes:
        admin:
        push:
        pull:
        maintain:
        triage:
    """

    admin: bool
    push: bool
    pull: bool
    maintain: Optional[bool] = None
    triage: Optional[bool] = None


@dataclass
class Organization(GitHubModel):
    """
    A GitHub organization

    Attributes:
        avatar_url: URL to the avatar image
        events_url: URL to the events
        followers_url: URL to the followers
        following_url: URL to users which the organization is following
        gists_url: URL to gists of the organization
        html_url: URL to the web page of the organization
        id: ID of the organization
        login: Login name of the organization
        node_id: Node ID of the organization
        organizations_url: URL to the organization
        received_events_url: URL to the received events
        repos_url: URL to the list of repositories
        site_admin:
        starred_url: URL to the list of starring users
        subscriptions_url:
        type: Type of the organization
        url: URL to the organization
        email: Email address
        gravatar_id: ID of the connected gravatar account
        name: Name of the organization
        starred_at:
    """

    avatar_url: str
    events_url: str
    followers_url: str
    following_url: str
    gists_url: str
    html_url: str
    id: int
    login: str
    node_id: str
    organizations_url: str
    received_events_url: str
    repos_url: str
    site_admin: bool
    starred_url: str
    subscriptions_url: str
    type: str
    url: str
    email: Optional[str] = None
    gravatar_id: Optional[str] = None
    name: Optional[str] = None
    starred_at: Optional[datetime] = None


@dataclass
class CodeOfConduct(GitHubModel):
    """
    Code of Conduct

    Attributes:
        url: URL to the code of conduct
        key: Key of the code of conduct
        name: Name of the code of conduct
        html_url: URL to the web page of the code of conduct
    """

    url: str
    key: str
    name: str
    html_url: str


class SecurityAndAnalysisStatus(Enum):
    """
    Security and analysis status

    Attributes:
        ENABLED: enabled
        DISABLED: disabled
    """

    ENABLED = "enabled"
    DISABLED = "disabled"


@dataclass
class SecurityAndAnalysisType(GitHubModel):
    """
    Security and analysis type

    Attributes:
        status:
    """

    status: SecurityAndAnalysisStatus


@dataclass
class SecurityAndAnalysis(GitHubModel):
    """
    Security and analysis

    Attributes:
        advanced_security: Status of GitHub Advanced Security is used
        secret_scanning: Status of Secret Scanning is used
        secret_scanning_push_protection: Status of Secret Scanning Push
            Protection is used
    """

    advanced_security: SecurityAndAnalysisType
    secret_scanning: SecurityAndAnalysisType
    secret_scanning_push_protection: SecurityAndAnalysisType


@dataclass
class Repository(GitHubModel):
    """
    A GitHub repository model

    Attributes:
        archive_url:
        assignees_url:
        blobs_url:
        branches_url:
        collaborators_url:
        comments_url:
        commits_url:
        compare_url:
        contents_url:
        contributors_url:
        deployments_url:
        downloads_url:
        events_url:
        fork:
        forks_url:
        full_name:
        git_commits_url:
        git_refs_url:
        git_tags_url:
        hooks_url:
        html_url:
        id:
        issue_comment_url:
        issue_events_url:
        issues_url:
        keys_url:
        labels_url:
        languages_url:
        merges_url:
        milestones_url:
        name:
        node_id:
        notifications_url:
        owner:
        private:
        pulls_url:
        releases_url:
        stargazers_url:
        statuses_url:
        subscribers_url:
        subscription_url:
        tags_url:
        teams_url:
        trees_url:
        url:
        allow_auto_merge:
        allow_forking:
        allow_merge_commit:
        allow_rebase_merge:
        allow_squash_merge:
        allow_update_branch:
        anonymous_access_enabled:
        archived:
        clone_url:
        code_of_conduct:
        created_at:
        default_branch:
        delete_branch_on_merge:
        description:
        disabled:
        forks_count:
        forks:
        git_url:
        has_discussions:
        has_downloads:
        has_issues:
        has_pages:
        has_projects:
        has_wiki:
        homepage:
        is_template:
        language:
        license:
        merge_commit_title:
        merge_commit_message:
        mirror_url:
        network_count:
        open_issues_count:
        open_issues:
        organization:
        permissions:
        pushed_at:
        security_and_analysis:
        size:
        ssh_url:
        stargazers_count:
        subscribers_count:
        svn_url:
        squash_merge_commit_message:
        squash_merge_commit_title:
        temp_clone_token:
        topics:
        updated_at: Last modification date
        use_squash_pr_title_as_default:
        visibility:
        watchers_count:
        watchers:
        web_commit_signoff_required:
    """

    archive_url: str
    assignees_url: str
    blobs_url: str
    branches_url: str
    collaborators_url: str
    comments_url: str
    commits_url: str
    compare_url: str
    contents_url: str
    contributors_url: str
    deployments_url: str
    downloads_url: str
    events_url: str
    fork: bool
    forks_url: str
    full_name: str
    git_commits_url: str
    git_refs_url: str
    git_tags_url: str
    hooks_url: str
    html_url: str
    id: int
    issue_comment_url: str
    issue_events_url: str
    issues_url: str
    keys_url: str
    labels_url: str
    languages_url: str
    merges_url: str
    milestones_url: str
    name: str
    node_id: str
    notifications_url: str
    owner: User
    private: bool
    pulls_url: str
    releases_url: str
    stargazers_url: str
    statuses_url: str
    subscribers_url: str
    subscription_url: str
    tags_url: str
    teams_url: str
    trees_url: str
    url: str
    allow_auto_merge: Optional[bool] = None
    allow_forking: Optional[bool] = None
    allow_merge_commit: Optional[bool] = None
    allow_rebase_merge: Optional[bool] = None
    allow_squash_merge: Optional[bool] = None
    allow_update_branch: Optional[bool] = None
    anonymous_access_enabled: Optional[bool] = None
    archived: Optional[bool] = None
    clone_url: Optional[str] = None
    code_of_conduct: Optional[CodeOfConduct] = None
    created_at: Optional[datetime] = None
    default_branch: Optional[str] = None
    delete_branch_on_merge: Optional[bool] = None
    description: Optional[str] = None
    disabled: Optional[bool] = None
    forks_count: Optional[int] = None
    forks: Optional[int] = None
    git_url: Optional[str] = None
    has_discussions: Optional[bool] = None
    has_downloads: Optional[bool] = None
    has_issues: Optional[bool] = None
    has_pages: Optional[bool] = None
    has_projects: Optional[bool] = None
    has_wiki: Optional[bool] = None
    homepage: Optional[str] = None
    is_template: Optional[bool] = None
    language: Optional[str] = None
    license: Optional[License] = None
    merge_commit_title: Optional[MergeCommitTitle] = None
    merge_commit_message: Optional[MergeCommitMessage] = None
    mirror_url: Optional[str] = None
    network_count: Optional[int] = None
    open_issues_count: Optional[int] = None
    open_issues: Optional[int] = None
    organization: Optional[Organization] = None
    permissions: Optional[RepositoryPermissions] = None
    pushed_at: Optional[datetime] = None
    security_and_analysis: Optional[SecurityAndAnalysis] = None
    size: Optional[int] = None
    ssh_url: Optional[str] = None
    stargazers_count: Optional[int] = None
    subscribers_count: Optional[int] = None
    svn_url: Optional[str] = None
    squash_merge_commit_message: Optional[SquashMergeCommitMessage] = None
    squash_merge_commit_title: Optional[SquashMergeCommitTitle] = None
    temp_clone_token: Optional[str] = None
    topics: Optional[List[str]] = field(default_factory=list[str])
    updated_at: Optional[datetime] = None
    use_squash_pr_title_as_default: Optional[bool] = None
    visibility: Optional[str] = None
    watchers_count: Optional[int] = None
    watchers: Optional[int] = None
    web_commit_signoff_required: Optional[bool] = None


class MemberFilter(Enum):
    """
    A member filter

    Attributes:
        TWO_FA_DISABLED: Members with 2 factor authentication disabled
        ALL: All members
    """

    TWO_FA_DISABLED = "2fa_disabled"
    ALL = "all"


class MemberRole(Enum):
    """
    A member role

    Attributes:
        ALL: All roles
        ADMIN: Admin only
        MEMBER: Member only
    """

    ALL = "all"
    ADMIN = "admin"
    MEMBER = "member"


class InvitationRole(Enum):
    """
    A invitation role

    Attributes:
        ADMIN: Admin only
        DIRECT_MEMBER: Direct member only
        BILLING_MANAGER: Billing manager only

    """

    ADMIN = "admin"
    DIRECT_MEMBER = "direct_member"
    BILLING_MANAGER = "billing_manager"


class GitIgnoreTemplate(Enum):
    """
    Just a small part of the available gitignore templates at
    https://github.com/github/gitignore

    Attributes:
        C: Template for C
        CPP: Template for C++
        CMAKE: Template for CMake
        GO: Template for Golang
        JAVA: Template for Java
        MAVEN: Template for maven
        NODE: Template for Nodejs
        PYTHON: Template for Python
        RUST: Template for Rust
    """

    C = "C"
    CPP = "C++"
    CMAKE = "CMake"
    GO = "Go"
    JAVA = "Java"
    MAVEN = "Maven"
    NODE = "Node"
    PYTHON = "Python"
    RUST = "Rust"


class LicenseType(Enum):
    """
    License Type

    Attributes:
        ACADEMIC_FREE_LICENSE_3_0: afl-3.0
        APACHE_LICENSE_2_0: apache-2.0
        ARTISTIC_LICENSE_2_0: artistic-2.0
        BOOST_SOFTWARE_LICENSE_1_0: bsl-1.0
        BSD_2_CLAUSE_SIMPLIFIED_LICENSE: bsd-2-clause
        BSD_3_CLAUSE_NEW_OR_REVISED_LICENSE: bsd-3-clause
        BSD_3_CLAUSE_CLEAR_LICENSE: bsd-3-clause-clear
        CREATIVE_COMMONS_LICENSE_FAMILY: cc
        CREATIVE_COMMONS_ZERO_1_0_UNIVERSAL: cc0-1.0
        CREATIVE_COMMONS_ATTRIBUTION_4_0: cc-by-4.0
        CREATIVE_COMMONS_ATTRIBUTION_SHARE_ALIKE_4_0: cc-by-sa-4.0
        DO_WHAT_THE_F_CK_YOU_WANT_TO_PUBLIC_LICENSE: wtfpl
        EDUCATIONAL_COMMUNITY_LICENSE_2_0: ecl-2.0
        ECLIPSE_PUBLIC_LICENSE_1_0: epl-1.0
        ECLIPSE_PUBLIC_LICENSE_2_0: epl-2.0
        EUROPEAN_UNION_PUBLIC_LICENSE_1_1: eupl-1.1
        GNU_AFFERO_GENERAL_PUBLIC_LICENSE_3_0: agpl-3.0
        GNU_GENERAL_PUBLIC_LICENSE_FAMILY: gpl
        GNU_GENERAL_PUBLIC_LICENSE_2_0: gpl-2.0
        GNU_GENERAL_PUBLIC_LICENSE_3_0: gpl-3.0
        GNU_LESSER_GENERAL_PUBLIC_LICENSE_FAMILY: lgpl
        GNU_LESSER_GENERAL_PUBLIC_LICENSE_2_1: lgpl-2.1
        GNU_LESSER_GENERAL_PUBLIC_LICENSE_3_0: lgpl-3.0
        ISC: isc
        LATEX_PROJECT_PUBLIC_LICENSE_1_3C_L: ppl-1.3c
        MICROSOFT_PUBLIC_LICENSE: ms-pl
        MIT: mit
        MOZILLA_PUBLIC_LICENSE_2_0: mpl-2.0
        OPEN_SOFTWARE_LICENSE_3_0: osl-3.0
        POSTGRESQL_LICENSE: postgresql
        SIL_OPEN_FONT_LICENSE_1_1: ofl-1.1
        UNIVERSITY_OF_ILLINOIS_NCSA_OPEN_SOURCE_LICENSE: ncsa
        THE_UNLICENSE: unlicense
        ZLIB_LICENSE: zlib
    """

    ACADEMIC_FREE_LICENSE_3_0 = "afl-3.0"
    APACHE_LICENSE_2_0 = "apache-2.0"
    ARTISTIC_LICENSE_2_0 = "artistic-2.0"
    BOOST_SOFTWARE_LICENSE_1_0 = "bsl-1.0"
    BSD_2_CLAUSE_SIMPLIFIED_LICENSE = "bsd-2-clause"
    BSD_3_CLAUSE_NEW_OR_REVISED_LICENSE = "bsd-3-clause"
    BSD_3_CLAUSE_CLEAR_LICENSE = "bsd-3-clause-clear"
    CREATIVE_COMMONS_LICENSE_FAMILY = "cc"
    CREATIVE_COMMONS_ZERO_1_0_UNIVERSAL = "cc0-1.0"
    CREATIVE_COMMONS_ATTRIBUTION_4_0 = "cc-by-4.0"
    CREATIVE_COMMONS_ATTRIBUTION_SHARE_ALIKE_4_0 = "cc-by-sa-4.0"
    DO_WHAT_THE_F_CK_YOU_WANT_TO_PUBLIC_LICENSE = "wtfpl"
    EDUCATIONAL_COMMUNITY_LICENSE_2_0 = "ecl-2.0"
    ECLIPSE_PUBLIC_LICENSE_1_0 = "epl-1.0"
    ECLIPSE_PUBLIC_LICENSE_2_0 = "epl-2.0"
    EUROPEAN_UNION_PUBLIC_LICENSE_1_1 = "eupl-1.1"
    GNU_AFFERO_GENERAL_PUBLIC_LICENSE_3_0 = "agpl-3.0"
    GNU_GENERAL_PUBLIC_LICENSE_FAMILY = "gpl"
    GNU_GENERAL_PUBLIC_LICENSE_2_0 = "gpl-2.0"
    GNU_GENERAL_PUBLIC_LICENSE_3_0 = "gpl-3.0"
    GNU_LESSER_GENERAL_PUBLIC_LICENSE_FAMILY = "lgpl"
    GNU_LESSER_GENERAL_PUBLIC_LICENSE_2_1 = "lgpl-2.1"
    GNU_LESSER_GENERAL_PUBLIC_LICENSE_3_0 = "lgpl-3.0"
    ISC = "isc"
    LATEX_PROJECT_PUBLIC_LICENSE_1_3C_L = "ppl-1.3c"
    MICROSOFT_PUBLIC_LICENSE = "ms-pl"
    MIT = "mit"
    MOZILLA_PUBLIC_LICENSE_2_0 = "mpl-2.0"
    OPEN_SOFTWARE_LICENSE_3_0 = "osl-3.0"
    POSTGRESQL_LICENSE = "postgresql"
    SIL_OPEN_FONT_LICENSE_1_1 = "ofl-1.1"
    UNIVERSITY_OF_ILLINOIS_NCSA_OPEN_SOURCE_LICENSE = "ncsa"
    THE_UNLICENSE = "unlicense"
    ZLIB_LICENSE = "zlib"



# Copyright (C) 2021-2023 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
import unittest
from pathlib import Path

from pontos.changelog.main import parse_args
from pontos.version.schemes._pep440 import PEP440Version


class ParseArgsTestCase(unittest.TestCase):
    def test_parse_args(self):
        parsed_args = parse_args(
            [
                "-q",
                "--project",
                "urghs",
                "--space",
                "bla",
                "--config",
                "foo.toml",
                "--current-version",
                "1.2.3",
                "--next-version",
                "2.3.4",
                "--git-tag-prefix",
                "a",
                "--output",
                "changelog.md",
            ]
        )

        self.assertTrue(parsed_args.quiet)
        self.assertEqual(parsed_args.project, "urghs")
        self.assertEqual(parsed_args.space, "bla")
        self.assertEqual(parsed_args.config, Path("foo.toml"))
        self.assertEqual(parsed_args.current_version, PEP440Version("1.2.3"))
        self.assertEqual(parsed_args.next_version, PEP440Version("2.3.4"))
        self.assertEqual(parsed_args.git_tag_prefix, "a")
        self.assertEqual(parsed_args.output, Path("changelog.md"))



# Copyright (C) 2022 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from dataclasses import dataclass
from datetime import datetime
from typing import Optional

from pontos.github.models.base import GitHubModel

__all__ = (
    "Artifact",
    "ArtifactWorkflowRun",
)


@dataclass
class ArtifactWorkflowRun(GitHubModel):
    """
    The workflow run that uploaded the artifact

    Attributes:
        id: ID of the workflow run
        repository_id: ID of the corresponding repository
        head_repository_id:
        head_branch: Corresponding branch name
        head_sha: Commit ID of the head of the corresponding branch
    """

    id: int
    repository_id: int
    head_repository_id: int
    head_branch: str
    head_sha: str


@dataclass
class Artifact(GitHubModel):
    """
    A GitHub Artifact model

    Attributes:
        id: ID of the artifact
        node_id: Node ID of the artifact
        name: Name of the artifact
        size_in_bytes: The size (in bytes) of the artifact
        url: REST API URL of the artifact
        archive_download_url: URL to download the artifact
        expired: True if the artifact has expired
        created_at: Creation date of the artifact
        expires_at: Expiration date of the artifact
        update_at: Last modification date of the artifact
        workflow_run: Corresponding GitHub workflow run
    """

    id: int
    node_id: str
    name: str
    size_in_bytes: int
    url: str
    archive_download_url: str
    expired: bool
    created_at: Optional[datetime] = None
    expires_at: Optional[datetime] = None
    updated_at: Optional[datetime] = None
    workflow_run: Optional[ArtifactWorkflowRun] = None



# Copyright (C) 2023 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import unittest
from contextlib import redirect_stderr
from io import StringIO
from pathlib import Path
from unittest.mock import patch

from pontos.release.helper import ReleaseType
from pontos.release.parser import DEFAULT_SIGNING_KEY, parse_args
from pontos.release.release import release
from pontos.release.sign import sign
from pontos.version.schemes._pep440 import PEP440Version


class ParseArgsTestCase(unittest.TestCase):
    def test_quiet(self):
        _, _, args = parse_args(["-q", "sign"])
        self.assertTrue(args.quiet)

        _, _, args = parse_args(["sign"])
        self.assertFalse(args.quiet)

    @patch.dict("os.environ", {"GITHUB_USER": "foo"})
    def test_user(self):
        user, _, _ = parse_args(["sign"])
        self.assertEqual(user, "foo")

    @patch.dict("os.environ", {"GITHUB_TOKEN": "foo"})
    def test_token(self):
        _, token, _ = parse_args(["sign"])
        self.assertEqual(token, "foo")


class ReleaseParseArgsTestCase(unittest.TestCase):
    def test_release_func(self):
        _, _, args = parse_args(["release", "--release-type", "patch"])

        self.assertEqual(args.func, release)

    def test_default(self):
        _, _, args = parse_args(["release", "--release-type", "patch"])

        self.assertEqual(args.git_tag_prefix, "v")
        self.assertEqual(args.space, "greenbone")
        self.assertFalse(args.local)

    def test_git_remote_name(self):
        _, _, args = parse_args(
            ["release", "--git-remote-name", "foo", "--release-type", "patch"]
        )

        self.assertEqual(args.git_remote_name, "foo")

    def test_git_signing_key(self):
        _, _, args = parse_args(
            ["release", "--git-signing-key", "123", "--release-type", "patch"]
        )

        self.assertEqual(args.git_signing_key, "123")

    def test_git_tag_prefix(self):
        _, _, args = parse_args(
            ["release", "--git-tag-prefix", "a", "--release-type", "patch"]
        )

        self.assertEqual(args.git_tag_prefix, "a")

        _, _, args = parse_args(
            ["release", "--git-tag-prefix", "", "--release-type", "patch"]
        )

        self.assertEqual(args.git_tag_prefix, "")

    def test_space(self):
        _, _, args = parse_args(
            ["release", "--space", "foo", "--release-type", "patch"]
        )

        self.assertEqual(args.space, "foo")

    def test_project(self):
        _, _, args = parse_args(
            ["release", "--project", "foo", "--release-type", "patch"]
        )

        self.assertEqual(args.project, "foo")

    def test_next_version(self):
        _, _, args = parse_args(
            ["release", "--next-version", "1.2.3", "--release-type", "patch"]
        )

        self.assertEqual(args.next_version, PEP440Version("1.2.3"))

    def test_release_type(self):
        _, _, args = parse_args(["release", "--release-type", "patch"])

        self.assertEqual(args.release_type, ReleaseType.PATCH)

        _, _, args = parse_args(["release", "--release-type", "calendar"])

        self.assertEqual(args.release_type, ReleaseType.CALENDAR)

        _, _, args = parse_args(["release", "--release-type", "minor"])

        self.assertEqual(args.release_type, ReleaseType.MINOR)

        _, _, args = parse_args(["release", "--release-type", "major"])

        self.assertEqual(args.release_type, ReleaseType.MAJOR)

        _, _, args = parse_args(["release", "--release-type", "alpha"])

        self.assertEqual(args.release_type, ReleaseType.ALPHA)

        _, _, args = parse_args(["release", "--release-type", "beta"])

        self.assertEqual(args.release_type, ReleaseType.BETA)

        _, _, args = parse_args(
            ["release", "--release-type", "release-candidate"]
        )

        self.assertEqual(args.release_type, ReleaseType.RELEASE_CANDIDATE)

        with self.assertRaises(SystemExit), redirect_stderr(StringIO()):
            parse_args(["release", "--release-type", "foo"])

    def test_release_type_version_without_release_version(self):
        with self.assertRaises(SystemExit), redirect_stderr(StringIO()):
            parse_args(["release", "--release-type", "version"])

        _, _, args = parse_args(
            [
                "release",
                "--release-type",
                "version",
                "--release-version",
                "1.2.3",
            ]
        )
        self.assertEqual(args.release_type, ReleaseType.VERSION)
        self.assertEqual(args.release_version, PEP440Version("1.2.3"))

    def test_release_version(self):
        _, _, args = parse_args(["release", "--release-version", "1.2.3"])

        self.assertEqual(args.release_version, PEP440Version("1.2.3"))
        self.assertEqual(args.release_type, ReleaseType.VERSION)

        with self.assertRaises(SystemExit), redirect_stderr(StringIO()):
            parse_args(["release", "--release-version", "1.2.3", "--patch"])

        with self.assertRaises(SystemExit), redirect_stderr(StringIO()):
            parse_args(["release", "--release-version", "1.2.3", "--calendar"])

        with self.assertRaises(SystemExit), redirect_stderr(StringIO()):
            parse_args(
                [
                    "release",
                    "--release-version",
                    "1.2.3",
                    "--release-type",
                    "patch",
                ]
            )

        with self.assertRaises(SystemExit), redirect_stderr(StringIO()):
            parse_args(
                [
                    "release",
                    "--release-version",
                    "1.2.3",
                    "--release-type",
                    "calendar",
                ]
            )

    def test_local(self):
        _, _, args = parse_args(
            ["release", "--local", "--release-type", "patch"]
        )

        self.assertTrue(args.local)

    def test_conventional_commits_config(self):
        _, _, args = parse_args(
            [
                "release",
                "--conventional-commits-config",
                "foo.toml",
                "--release-type",
                "patch",
            ]
        )

        self.assertEqual(args.cc_config, Path("foo.toml"))

    def test_release_series(self):
        _, _, args = parse_args(
            ["release", "--release-type", "patch", "--release-series", "22.4"]
        )

        self.assertEqual(args.release_series, "22.4")

    def test_update_project(self):
        _, _, args = parse_args(["release", "--release-type", "patch"])

        self.assertTrue(args.update_project)

        _, _, args = parse_args(
            ["release", "--release-type", "patch", "--update-project"]
        )

        self.assertTrue(args.update_project)

        _, _, args = parse_args(
            ["release", "--release-type", "patch", "--no-update-project"]
        )

        self.assertFalse(args.update_project)


class SignParseArgsTestCase(unittest.TestCase):
    def test_sign_func(self):
        _, _, args = parse_args(["sign"])

        self.assertEqual(args.func, sign)

    def test_default(self):
        _, _, args = parse_args(["sign"])

        self.assertEqual(args.git_tag_prefix, "v")
        self.assertEqual(args.space, "greenbone")
        self.assertEqual(args.signing_key, DEFAULT_SIGNING_KEY)

    def test_space(self):
        _, _, args = parse_args(["sign", "--space", "foo"])

        self.assertEqual(args.space, "foo")

    def test_project(self):
        _, _, args = parse_args(["sign", "--project", "foo"])

        self.assertEqual(args.project, "foo")

    def test_release_version(self):
        _, _, args = parse_args(["sign", "--release-version", "1.2.3"])

        self.assertEqual(args.release_version, PEP440Version("1.2.3"))

    def test_dry_run(self):
        _, _, args = parse_args(["sign", "--dry-run"])

        self.assertTrue(args.dry_run)

    def test_signing_key(self):
        _, _, args = parse_args(["sign", "--signing-key", "123"])

        self.assertEqual(args.signing_key, "123")

    def test_passphrase(self):
        _, _, args = parse_args(["sign", "--passphrase", "123"])

        self.assertEqual(args.passphrase, "123")

    def test_release_series(self):
        _, _, args = parse_args(["sign", "--release-series", "22.4"])

        self.assertEqual(args.release_series, "22.4")



# -*- coding: utf-8 -*-
# pontos/release/release.py
# Copyright (C) 2020-2022 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import subprocess
import sys
from typing import NoReturn

from pontos.git import GitError
from pontos.release.parser import parse_args
from pontos.terminal.null import NullTerminal
from pontos.terminal.rich import RichTerminal


def main(
    args=None,
) -> NoReturn:
    username, token, parsed_args = parse_args(args)
    if parsed_args.quiet:
        term = NullTerminal()
    else:
        term = RichTerminal()

    term.bold_info(f"pontos-release => {parsed_args.func.__name__}")

    with term.indent():
        try:
            retval = parsed_args.func(
                term,
                parsed_args,
                username=username,
                token=token,
            )
            sys.exit(int(retval))
        except KeyboardInterrupt:
            sys.exit(1)
        except GitError as e:
            term.error(f'Could not run git command "{e.cmd}".')
            error = e.stderr if e.stderr else e.stdout
            term.print(f"Output was: {error}")
            sys.exit(1)
        except subprocess.CalledProcessError as e:
            if not "--passphrase" in e.cmd:
                term.error(f'Could not run command "{e.cmd}".')
            else:
                term.error("Headless signing failed.")

            term.print(f"Error was: {e.stderr}")
            sys.exit(1)


if __name__ == "__main__":
    main()



# Copyright (C) 2019-2022 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# pylint: disable=invalid-name, protected-access

import unittest
from io import StringIO
from unittest.mock import MagicMock, patch

import colorful as cf

from pontos.terminal.terminal import ConsoleTerminal as Terminal
from pontos.terminal.terminal import Signs


class TerminalTestCase(unittest.TestCase):
    def setUp(self):
        self.maxDiff = 180
        # getting the bash-color-codes from the colorful module
        self.red = cf.red
        self.green = cf.green
        self.yellow = cf.yellow
        self.cyan = cf.cyan
        self.white = cf.white
        self.reset = cf.reset
        self.bold = cf.bold
        # every colors second value is the reset value ...
        self.term = Terminal()
        self.term.get_width = MagicMock(return_value=80)

    @patch("sys.stdout", new_callable=StringIO)
    def test_error(self, mock_stdout):
        status = f"{self.red(Signs.ERROR)} "
        msg = "foo bar"

        expected_msg = self.reset(f"{status}{msg}").styled_string + "\n"
        expected_len = len(expected_msg)

        self.term.error(msg)

        ret = mock_stdout.getvalue()

        self.assertEqual(ret, expected_msg)
        self.assertEqual(len(ret), expected_len)

    @patch("sys.stdout", new_callable=StringIO)
    def test_fail(self, mock_stdout):
        status = f"{self.red(Signs.FAIL)} "
        msg = "foo bar baz"

        expected_msg = self.reset(f"{status}{msg}").styled_string + "\n"
        expected_len = len(expected_msg)

        self.term.fail(msg)

        ret = mock_stdout.getvalue()

        self.assertEqual(ret, expected_msg)
        self.assertEqual(len(ret), expected_len)

    @patch("sys.stdout", new_callable=StringIO)
    def test_info(self, mock_stdout):
        status = f"{self.cyan(Signs.INFO)} "
        msg = "foo bar"

        expected_msg = self.reset(f"{status}{msg}").styled_string + "\n"
        expected_len = len(expected_msg)

        self.term.info(msg)

        ret = mock_stdout.getvalue()

        self.assertEqual(ret, expected_msg)
        self.assertEqual(len(ret), expected_len)

    @patch("sys.stdout", new_callable=StringIO)
    def test_info_with_newline(self, mock_stdout):
        status = f"{self.cyan(Signs.INFO)} "
        msg = "foo bar\nbaz"
        repl_msg = msg.replace("\n", "\n  ")

        expected_msg = self.reset(f"{status}{repl_msg}").styled_string + "\n"
        expected_len = len(expected_msg)

        self.term.info(msg)

        ret = mock_stdout.getvalue()

        self.assertEqual(ret, expected_msg)
        self.assertEqual(len(ret), expected_len)

    @patch("sys.stdout", new_callable=StringIO)
    def test_bold_info(self, mock_stdout):
        status = f"{self.cyan(Signs.INFO)} "
        msg = "bold foo bar"

        expected_msg = self.bold(f"{status}{msg}").styled_string + "\n"
        expected_len = len(expected_msg)

        self.term.bold_info(msg)

        ret = mock_stdout.getvalue()

        self.assertEqual(ret, expected_msg)
        self.assertEqual(len(ret), expected_len)

    @patch("sys.stdout", new_callable=StringIO)
    def test_ok(self, mock_stdout):
        status = f"{self.green(Signs.OK)} "
        msg = "foo bar"

        expected_msg = self.reset(f"{status}{msg}").styled_string + "\n"
        expected_len = len(expected_msg)

        self.term.ok(msg)

        ret = mock_stdout.getvalue()

        self.assertEqual(ret, expected_msg)
        self.assertEqual(len(ret), expected_len)

    @patch("sys.stdout", new_callable=StringIO)
    def test_warning(self, mock_stdout):
        msg = "foo bar"

        status = f"{self.yellow(Signs.WARNING)} "

        expected_msg = self.reset(f"{status}{msg}").styled_string + "\n"
        expected_len = len(expected_msg)

        self.term.warning(msg)

        ret = mock_stdout.getvalue()

        self.assertEqual(ret, expected_msg)
        self.assertEqual(len(ret), expected_len)

    @patch("sys.stdout", new_callable=StringIO)
    def test_print(self, mock_stdout):
        expected_msg = (
            self.reset(f'{self.white(" ")} foo bar').styled_string + "\n"
        )

        self.term.print("foo bar")

        ret = mock_stdout.getvalue()

        self.assertEqual(len(ret), len(expected_msg))
        self.assertEqual(ret, expected_msg)

    @patch("sys.stdout", new_callable=StringIO)
    def test_with_indent(self, mock_stdout):
        expected_msg = (
            self.reset(f'{self.white(" ")}   foo').styled_string + "\n"
        )

        with self.term.indent(2):
            self.term.print("foo")

            ret = mock_stdout.getvalue()

        self.assertEqual(len(ret), len(expected_msg))
        self.assertEqual(ret, expected_msg)

        # clear the buffer
        mock_stdout.truncate(0)
        mock_stdout.seek(0)

        expected_msg = self.reset(f'{self.white(" ")} bar').styled_string + "\n"
        self.term.print("bar")

        ret = mock_stdout.getvalue()

        self.assertEqual(len(ret), len(expected_msg))
        self.assertEqual(ret, expected_msg)

    @patch("sys.stdout", new_callable=StringIO)
    def test_long_msg(self, mock_stdout):
        long_msg = (
            "Lorem ipsum dolor sit amet, consetetur sadipscing elitr, "
            "sed diam nonumy eirmod tempor invidunt ut labore et dolore magna"
            " aliquyam erat, sed diam voluptua."
        )
        expected_msg = (
            self.reset(
                f'{self.white(" ")} Lorem ipsum dolor sit amet, consetetur '
                "sadipscing elitr, sed diam nonumy eirmo\n  d tempor invidunt "
                "ut labore et dolore magna aliquyam erat, sed diam voluptua."
            ).styled_string
            + "\n"
        )
        expected_len = len(expected_msg)

        self.term.print(long_msg)

        ret = mock_stdout.getvalue()

        self.assertEqual(ret, expected_msg)
        self.assertEqual(len(ret), expected_len)


if __name__ == "__main__":
    unittest.main()



# Copyright (C) 2023 Greenbone Networks GmbH
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from argparse import ArgumentTypeError

from ._pep440 import PEP440VersioningScheme
from ._scheme import VersioningScheme
from ._semantic import SemanticVersioningScheme

__all__ = (
    "VERSIONING_SCHEMES",
    "versioning_scheme_argument_type",
    "VersioningScheme",
    "PEP440VersioningScheme",
    "SemanticVersioningScheme",
)

#: Dictionary with available versioning schemes
VERSIONING_SCHEMES = {
    "pep440": PEP440VersioningScheme,
    "semver": SemanticVersioningScheme,
}


def versioning_scheme_argument_type(value: str) -> VersioningScheme:
    """
    Verifies if the passed value is a valid versioning scheme and returns
    the corresponding versioning scheme.

    Intended to be used as in `ArgumentParser.add_argument` as the type.

    Raises:
        ArgumentTypeError: If the passed value is not a valid versioning scheme

    Example:
        .. code-block:: python

            from argparse import ArgumentParser
            from pontos.version.scheme versioning_scheme_argument_type

            parser = ArgumentParser()
            parser.add_argument(
                "--versioning-scheme",
                type=versioning_scheme_argument_type,
            )
    """
    try:
        return VERSIONING_SCHEMES[value]
    except KeyError:
        raise ArgumentTypeError(
            f"invalid value {value}. Expected one of "
            f"{', '.join(VERSIONING_SCHEMES.keys())}."
        ) from None



# Copyright (C) 2022 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# pylint: disable=redefined-builtin, line-too-long, too-many-lines

from pathlib import Path
from unittest.mock import MagicMock, call

import httpx

from pontos.github.api.release import GitHubAsyncRESTReleases
from tests import AsyncIteratorMock, AsyncMock, aiter, anext
from tests.github.api import GitHubAsyncRESTTestCase, create_response

here = Path(__file__).parent

RELEASE_JSON = {
    "url": "https://api.github.com/repos/octocat/Hello-World/releases/1",
    "html_url": "https://github.com/octocat/Hello-World/releases/v1.0.0",
    "assets_url": "https://api.github.com/repos/octocat/Hello-World/releases/1/assets",
    "upload_url": "https://uploads.github.com/repos/octocat/Hello-World/releases/1/assets{?name,label}",
    "tarball_url": "https://api.github.com/repos/octocat/Hello-World/tarball/v1.0.0",
    "zipball_url": "https://api.github.com/repos/octocat/Hello-World/zipball/v1.0.0",
    "discussion_url": "https://github.com/octocat/Hello-World/discussions/90",
    "id": 1,
    "node_id": "MDc6UmVsZWFzZTE=",
    "tag_name": "v1.0.0",
    "target_commitish": "master",
    "name": "v1.0.0",
    "body": "Description of the release",
    "draft": False,
    "prerelease": False,
    "created_at": "2013-02-27T19:35:32Z",
    "published_at": "2013-02-27T19:35:32Z",
    "author": {
        "login": "octocat",
        "id": 1,
        "node_id": "MDQ6VXNlcjE=",
        "avatar_url": "https://github.com/images/error/octocat_happy.gif",
        "gravatar_id": "",
        "url": "https://api.github.com/users/octocat",
        "html_url": "https://github.com/octocat",
        "followers_url": "https://api.github.com/users/octocat/followers",
        "following_url": "https://api.github.com/users/octocat/following{/other_user}",
        "gists_url": "https://api.github.com/users/octocat/gists{/gist_id}",
        "starred_url": "https://api.github.com/users/octocat/starred{/owner}{/repo}",
        "subscriptions_url": "https://api.github.com/users/octocat/subscriptions",
        "organizations_url": "https://api.github.com/users/octocat/orgs",
        "repos_url": "https://api.github.com/users/octocat/repos",
        "events_url": "https://api.github.com/users/octocat/events{/privacy}",
        "received_events_url": "https://api.github.com/users/octocat/received_events",
        "type": "User",
        "site_admin": False,
    },
    "assets": [
        {
            "url": "https://api.github.com/repos/octocat/Hello-World/releases/assets/1",
            "browser_download_url": "https://github.com/octocat/Hello-World/releases/download/v1.0.0/example.zip",
            "id": 1,
            "node_id": "MDEyOlJlbGVhc2VBc3NldDE=",
            "name": "example.zip",
            "label": "short description",
            "state": "uploaded",
            "content_type": "application/zip",
            "size": 1024,
            "download_count": 42,
            "created_at": "2013-02-27T19:35:32Z",
            "updated_at": "2013-02-27T19:35:32Z",
            "uploader": {
                "login": "octocat",
                "id": 1,
                "node_id": "MDQ6VXNlcjE=",
                "avatar_url": "https://github.com/images/error/octocat_happy.gif",
                "gravatar_id": "",
                "url": "https://api.github.com/users/octocat",
                "html_url": "https://github.com/octocat",
                "followers_url": "https://api.github.com/users/octocat/followers",
                "following_url": "https://api.github.com/users/octocat/following{/other_user}",
                "gists_url": "https://api.github.com/users/octocat/gists{/gist_id}",
                "starred_url": "https://api.github.com/users/octocat/starred{/owner}{/repo}",
                "subscriptions_url": "https://api.github.com/users/octocat/subscriptions",
                "organizations_url": "https://api.github.com/users/octocat/orgs",
                "repos_url": "https://api.github.com/users/octocat/repos",
                "events_url": "https://api.github.com/users/octocat/events{/privacy}",
                "received_events_url": "https://api.github.com/users/octocat/received_events",
                "type": "User",
                "site_admin": False,
            },
        }
    ],
}


class GitHubAsyncRESTReleasesTestCase(GitHubAsyncRESTTestCase):
    api_cls = GitHubAsyncRESTReleases

    async def test_exists(self):
        response = create_response(is_success=True)
        self.client.get.return_value = response

        self.assertTrue(await self.api.exists("foo/bar", "v1.2.3"))

        self.client.get.assert_awaited_once_with(
            "/repos/foo/bar/releases/tags/v1.2.3"
        )

    async def test_not_exists(self):
        response = create_response(is_success=False)
        self.client.get.return_value = response

        self.assertFalse(await self.api.exists("foo/bar", "v1.2.3"))

        self.client.get.assert_awaited_once_with(
            "/repos/foo/bar/releases/tags/v1.2.3"
        )

    async def test_create(self):
        response = create_response()
        response.json.return_value = RELEASE_JSON
        self.client.post.return_value = response

        release = await self.api.create(
            "foo/bar",
            "v1.2.3",
            body="foo",
            name="baz",
            target_commitish="stable",
        )

        self.client.post.assert_awaited_once_with(
            "/repos/foo/bar/releases",
            data={
                "tag_name": "v1.2.3",
                "draft": False,
                "prerelease": False,
                "name": "baz",
                "body": "foo",
                "target_commitish": "stable",
            },
        )

        self.assertEqual(release.id, 1)

    async def test_create_simple(self):
        response = create_response()
        response.json.return_value = RELEASE_JSON
        self.client.post.return_value = response

        release = await self.api.create(
            "foo/bar",
            "v1.2.3",
        )

        self.client.post.assert_awaited_once_with(
            "/repos/foo/bar/releases",
            data={
                "tag_name": "v1.2.3",
                "draft": False,
                "prerelease": False,
            },
        )

        self.assertEqual(release.id, 1)

    async def test_create_failure(self):
        response = create_response()
        self.client.post.side_effect = httpx.HTTPStatusError(
            "404", request=MagicMock(), response=response
        )

        with self.assertRaises(httpx.HTTPStatusError):
            await self.api.create(
                "foo/bar",
                "v1.2.3",
                body="foo",
                name="baz",
                target_commitish="stable",
            )

        self.client.post.assert_awaited_once_with(
            "/repos/foo/bar/releases",
            data={
                "tag_name": "v1.2.3",
                "draft": False,
                "prerelease": False,
                "name": "baz",
                "body": "foo",
                "target_commitish": "stable",
            },
        )

    async def test_get(self):
        response = create_response()
        response.json.return_value = RELEASE_JSON
        self.client.get.return_value = response

        release = await self.api.get(
            "foo/bar",
            "v1.2.3",
        )

        self.client.get.assert_awaited_once_with(
            "/repos/foo/bar/releases/tags/v1.2.3",
        )
        self.assertEqual(release.id, 1)

    async def test_get_failure(self):
        response = create_response()
        self.client.get.side_effect = httpx.HTTPStatusError(
            "404", request=MagicMock(), response=response
        )

        with self.assertRaises(httpx.HTTPStatusError):
            await self.api.get(
                "foo/bar",
                "v1.2.3",
            )

        self.client.get.assert_awaited_once_with(
            "/repos/foo/bar/releases/tags/v1.2.3",
        )

    async def test_download_release_tarball(self):
        response = create_response(headers=MagicMock())
        response.headers.get.return_value = 2
        response.aiter_bytes.return_value = AsyncIteratorMock(["1", "2"])
        stream_context = AsyncMock()
        stream_context.__aenter__.return_value = response
        self.client.stream.return_value = stream_context

        async with self.api.download_release_tarball(
            "foo/bar", "v1.2.3"
        ) as download_iterable:
            it = aiter(download_iterable)
            content, progress = await anext(it)

            self.assertEqual(content, "1")
            self.assertEqual(progress, 50)

            content, progress = await anext(it)
            self.assertEqual(content, "2")
            self.assertEqual(progress, 100)

        self.client.stream.assert_called_once_with(
            "https://github.com/foo/bar/archive/refs/tags/v1.2.3.tar.gz"
        )

    async def test_download_release_zip(self):
        response = create_response(headers=MagicMock())
        response.headers.get.return_value = 2
        response.aiter_bytes.return_value = AsyncIteratorMock(["1", "2"])
        stream_context = AsyncMock()
        stream_context.__aenter__.return_value = response
        self.client.stream.return_value = stream_context

        async with self.api.download_release_zip(
            "foo/bar", "v1.2.3"
        ) as download_iterable:
            it = aiter(download_iterable)
            content, progress = await anext(it)

            self.assertEqual(content, "1")
            self.assertEqual(progress, 50)

            content, progress = await anext(it)
            self.assertEqual(content, "2")
            self.assertEqual(progress, 100)

        self.client.stream.assert_called_once_with(
            "https://github.com/foo/bar/archive/refs/tags/v1.2.3.zip"
        )

    async def test_download_release_assets(self):
        get_assets_url_response = create_response()
        data = RELEASE_JSON.copy()
        data.update({"assets_url": "https://foo.bar/assets"})
        get_assets_url_response.json.return_value = data
        get_assets_response = create_response()
        get_assets_response.json.return_value = [
            {"url": "http://bar", "name": "bar"},
            {"url": "http://baz", "name": "baz"},
        ]
        response = create_response(headers=MagicMock())
        response.headers.get.return_value = 2
        response.aiter_bytes.return_value = AsyncIteratorMock(["1", "2"])
        stream_context = AsyncMock()
        stream_context.__aenter__.return_value = response
        self.client.stream.return_value = stream_context
        self.client.get.side_effect = [
            get_assets_url_response,
            get_assets_response,
        ]

        assets_it = aiter(self.api.download_release_assets("foo/bar", "v1.2.3"))

        name, cm = await anext(assets_it)

        self.client.get.assert_has_awaits(
            [
                call("/repos/foo/bar/releases/tags/v1.2.3"),
                call("https://foo.bar/assets"),
            ]
        )

        self.client.stream.assert_called_once_with("http://bar")

        self.assertEqual(name, "bar")

        async with cm as progress_it:
            it = aiter(progress_it)
            content, progress = await anext(it)

            self.assertEqual(content, "1")
            self.assertEqual(progress, 50)

            content, progress = await anext(it)
            self.assertEqual(content, "2")
            self.assertEqual(progress, 100)

        self.client.stream.reset_mock()
        response = create_response(headers=MagicMock())
        response.headers.get.return_value = 2
        response.aiter_bytes.return_value = AsyncIteratorMock(["1", "2"])
        stream_context = AsyncMock()
        stream_context.__aenter__.return_value = response
        self.client.stream.return_value = stream_context

        name, cm = await anext(assets_it)

        self.client.stream.assert_called_once_with("http://baz")

        self.assertEqual(name, "baz")

        async with cm as progress_it:
            it = aiter(progress_it)
            content, progress = await anext(it)

            self.assertEqual(content, "1")
            self.assertEqual(progress, 50)

            content, progress = await anext(it)
            self.assertEqual(content, "2")
            self.assertEqual(progress, 100)

        with self.assertRaises(StopAsyncIteration):
            await anext(assets_it)

    async def test_download_release_assets_no_assets_url(self):
        get_assets_url_response = create_response()
        data = RELEASE_JSON.copy()
        data.update({"assets_url": None})
        get_assets_url_response.json.return_value = data
        self.client.get.return_value = get_assets_url_response
        assets_it = aiter(self.api.download_release_assets("foo/bar", "v1.2.3"))

        with self.assertRaises(RuntimeError):
            await anext(assets_it)

        self.client.get.assert_awaited_once_with(
            "/repos/foo/bar/releases/tags/v1.2.3"
        )

    async def test_download_release_assets_filter(self):
        get_assets_url_response = create_response()
        data = RELEASE_JSON.copy()
        data.update({"assets_url": "https://foo.bar/assets"})
        get_assets_url_response.json.return_value = data
        get_assets_response = create_response()
        get_assets_response.json.return_value = [
            {"url": "http://bar", "name": "bar"},
            {"url": "http://baz", "name": "baz"},
        ]
        response = create_response(headers=MagicMock())
        response.headers.get.return_value = 2
        response.aiter_bytes.return_value = AsyncIteratorMock(["1", "2"])
        stream_context = AsyncMock()
        stream_context.__aenter__.return_value = response
        self.client.stream.return_value = stream_context
        self.client.get.side_effect = [
            get_assets_url_response,
            get_assets_response,
        ]

        assets_it = aiter(
            self.api.download_release_assets(
                "foo/bar", "v1.2.3", match_pattern="*r"
            )
        )

        name, cm = await anext(assets_it)

        self.client.get.assert_has_awaits(
            [
                call("/repos/foo/bar/releases/tags/v1.2.3"),
                call("https://foo.bar/assets"),
            ]
        )

        self.client.stream.assert_called_once_with("http://bar")

        self.assertEqual(name, "bar")

        async with cm as progress_it:
            it = aiter(progress_it)
            content, progress = await anext(it)

            self.assertEqual(content, "1")
            self.assertEqual(progress, 50)

            content, progress = await anext(it)
            self.assertEqual(content, "2")
            self.assertEqual(progress, 100)

        with self.assertRaises(StopAsyncIteration):
            await anext(assets_it)

    async def test_upload_release_assets(self):
        response = create_response()
        data = RELEASE_JSON.copy()
        data.update({"upload_url": "https://uploads/assets{?name,label}"})
        response.json.return_value = data
        post_response = create_response()
        self.client.get.return_value = response
        self.client.post.return_value = post_response

        file1 = MagicMock(spec=Path)
        file1.name = "foo.txt"
        content1 = b"foo"
        file1.open.return_value.__enter__.return_value.read.side_effect = [
            content1
        ]
        file2 = MagicMock(spec=Path)
        file2.name = "bar.pdf"
        content2 = b"bar"
        file2.open.return_value.__enter__.return_value.read.side_effect = [
            content2
        ]
        upload_files = [file1, (file2, "application/pdf")]

        def assert_file1(index: int):
            args = self.client.post.await_args_list[index].args
            self.assertEqual(args, ("https://uploads/assets",))
            kwargs = self.client.post.await_args_list[index].kwargs
            self.assertEqual(kwargs["params"], {"name": "foo.txt"})
            self.assertEqual(kwargs["content_type"], "application/octet-stream")

        def assert_file2(index: int):
            args = self.client.post.await_args_list[index].args
            self.assertEqual(args, ("https://uploads/assets",))
            kwargs = self.client.post.await_args_list[index].kwargs
            self.assertEqual(kwargs["params"], {"name": "bar.pdf"})
            self.assertEqual(kwargs["content_type"], "application/pdf")

        it = aiter(
            self.api.upload_release_assets("foo/bar", "v1.2.3", upload_files)
        )

        # the order of the files is non-deterministic

        f = await anext(it)
        if f == file1:
            assert_file1(0)
        else:
            assert_file2(0)

        f = await anext(it)
        if f == file1:
            assert_file1(1)
        else:
            assert_file2(1)

        self.client.get.assert_awaited_once_with(
            "/repos/foo/bar/releases/tags/v1.2.3"
        )
